@@ -63,6 +63,8 @@ typedef enum {
 	XT_DESCRIPTION,
 	XT_EXCEL_TIME,
 	XT_FACILITY,
+	XT_FILENAME,
+	XT_FORMAT,
 	XT_GEOCACHE_CONTAINER,
 	XT_GEOCACHE_DIFF,
 	XT_GEOCACHE_HINT,
@@ -93,6 +95,7 @@ typedef enum {
 	XT_LAT_DIRDECIMAL,
 	XT_LAT_HUMAN_READABLE,
 	XT_LAT_INT32DEG,
+	XT_LAT_DDMMDIR,
 	XT_LAT_NMEA,
 	XT_LOCAL_TIME,
 	XT_LON_DECIMAL,
@@ -101,9 +104,11 @@ typedef enum {
 	XT_LON_DIRDECIMAL,
 	XT_LON_HUMAN_READABLE,
 	XT_LON_INT32DEG,
+	XT_LON_DDMMDIR,
 	XT_LON_NMEA,
 	XT_MAP_EN_BNG,
 	XT_NOTES,
+	XT_NET_TIME,
 	XT_PATH_COURSE,
 	XT_PATH_DISTANCE_KM,
 	XT_PATH_DISTANCE_MILES,
@@ -119,7 +124,14 @@ typedef enum {
 	XT_STREET_ADDR,
 	XT_TIMET_TIME,
 	XT_TRACK_NAME,
+	XT_TRACK_NEW,
 	XT_URL,
+        XT_UTM,
+        XT_UTM_ZONE,
+        XT_UTM_ZONEC,
+        XT_UTM_ZONEF,
+        XT_UTM_EASTING,
+        XT_UTM_NORTHING,
 	XT_URL_LINK_TEXT,
 	XT_YYYYMMDD_TIME
 } xcsv_token;
@@ -134,15 +146,17 @@ xcsv_file_t xcsv_file;
 extern char *xcsv_urlbase;
 extern char *prefer_shortnames;
 
-extern geocache_type gs_mktype(const char *t);
-extern geocache_container gs_mkcont(const char *t);
-
+#if CSVFMTS_ENABLED
 static double pathdist = 0;
 static double oldlon = 999;
 static double oldlat = 999;
-    
+
 static int waypt_out_count;
 static route_head *csv_track, *csv_route;
+#endif // CSVFMTS_ENABLED
+
+static double utm_northing, utm_easting, utm_zone = 0;
+static char utm_zonec;
 
 /*********************************************************************/
 /* csv_stringclean() - remove any unwanted characters from string.   */
@@ -341,7 +355,7 @@ csv_lineparse(const char *stringstart, const char *delimited_by,
     }
     
     /* allocate enough space for this data field */
-    tmp = xcalloc((p - sp) + 1, sizeof(char));
+    tmp = (char *) xcalloc((p - sp) + 1, sizeof(char));
 
     strncpy(tmp, sp, (p - sp));
     tmp[p - sp] = '\0'; 
@@ -438,6 +452,24 @@ decdir_to_dec(const char * decdir)
     
     return(rval * sign);
 }
+
+/*****************************************************************************/
+/* ddmmdir_to_degrees() - convert ddmm/direction value into degrees          */
+/* usage: lat = ddmmdir_to_degrees("W90.1234");                              */
+/*        lat = ddmmdir_to_degrees("30.1234N");                              */
+/*****************************************************************************/
+static double
+ddmmdir_to_degrees(const char * ddmmdir)
+{
+      // if not N or E, prepend a '-' to ddmm2degrees input
+      // see XT_LAT_NMEA which handles ddmm directly
+      if (strchr(ddmmdir, 'W') || strchr(ddmmdir, 'S'))
+      {
+         return ddmm2degrees(- atof(ddmmdir));
+      }
+      return ddmm2degrees(atof(ddmmdir));
+
+}
 #endif
 
 /*****************************************************************************
@@ -598,7 +630,7 @@ dec_to_human( char *buff, const char *format, const char *dirs, double val )
     dblvals[2] = 60*(dblvals[1]-intvals[1]);
     intvals[2] = (int)dblvals[2];
     
-    subformat = xmalloc( strlen(format)+2);
+    subformat = (char*) xmalloc( strlen(format)+2);
     formatptr = format;
     
     buff[0] = '\0';
@@ -633,7 +665,7 @@ dec_to_human( char *buff, const char *format, const char *dirs, double val )
 		    index++;
 		    break;
 		case '%':
-		    sprintf( buff+strlen(buff), subformat );
+		    sprintf( buff+strlen(buff), "%s", subformat );
 		    break;
 		default:
 		    fatal(MYNAME ": invalid format specifier\n");
@@ -642,7 +674,7 @@ dec_to_human( char *buff, const char *format, const char *dirs, double val )
 	    }
 	}
 	else {
-	    sprintf( buff+strlen(buff), subformat );
+	    sprintf( buff+strlen(buff), "%s", subformat );
 	}
 	formatptr += strlen(subformat);
     } 
@@ -665,14 +697,14 @@ xcsv_file_init(void)
     /* ofield is alloced to allow pointing back at ifields
      * where applicable.
      */
-    xcsv_file.ofield = xcalloc(sizeof(queue), 1);
+    xcsv_file.ofield = (queue*) xcalloc(sizeof(queue), 1);
     QUEUE_INIT(xcsv_file.ofield);
     /*
      * Provide a sane default for CSV _files_.
      */
     xcsv_file.type = ff_type_file;
 
-    xcsv_file.mkshort_handle = mkshort_new_handle();
+    xcsv_file.mkshort_handle = (struct short_handle_*) mkshort_new_handle();
     xcsv_file.gps_datum = GPS_DATUM_WGS84;
 }
 
@@ -683,7 +715,7 @@ xcsv_file_init(void)
 void
 xcsv_ifield_add(char *key, char *val, char *pfc)
 {
-    field_map_t *fmp = xcalloc(sizeof(*fmp), 1);
+    field_map_t *fmp = (field_map_t *) xcalloc(sizeof(*fmp), 1);
     struct xt_mapping *xm = in_word_set(key, strlen(key));
     
     fmp->key = key;
@@ -702,7 +734,7 @@ xcsv_ifield_add(char *key, char *val, char *pfc)
 void
 xcsv_ofield_add(char *key, char *val, char *pfc, int options)
 {
-    field_map_t *fmp = xcalloc(sizeof(*fmp), 1);
+    field_map_t *fmp = (field_map_t *) xcalloc(sizeof(*fmp), 1);
     struct xt_mapping *xm = in_word_set(key, strlen(key));
     
     fmp->key = key;
@@ -722,7 +754,7 @@ xcsv_ofield_add(char *key, char *val, char *pfc, int options)
 void
 xcsv_prologue_add(char *prologue)
 {
-    ogue_t* ogp = xcalloc(sizeof(*ogp), 1);
+    ogue_t* ogp = (ogue_t*) xcalloc(sizeof(*ogp), 1);
 
     ogp->val = prologue;
     ENQUEUE_TAIL(&xcsv_file.prologue, &ogp->Q);
@@ -736,7 +768,7 @@ xcsv_prologue_add(char *prologue)
 void
 xcsv_epilogue_add(char *epilogue)
 {
-    ogue_t * ogp = xcalloc(sizeof(*ogp), 1);
+    ogue_t * ogp = (ogue_t*) xcalloc(sizeof(*ogp), 1);
 
     ogp->val = epilogue;
     ENQUEUE_TAIL(&xcsv_file.epilogue, &ogp->Q);
@@ -757,7 +789,11 @@ yyyymmdd_to_time(const char *s)
 	tm.tm_mon = t % 100 - 1;
 	t = t / 100;
 	tm.tm_year = t - 1900;
-	return mktime(&tm);
+	
+	if (mkgmtime(&tm) > 0)
+		return mktime(&tm);
+	else
+		return 0;
 }
 
 
@@ -804,7 +840,7 @@ addhms( const char *s, const char *format )
 	char * ampm = NULL;
 	int ac;
 	
-	ampm = xmalloc( strlen(s) );
+	ampm = (char*) xmalloc( strlen(s) );
 	ac = sscanf(s, format, &hour, &min, &sec, &ampm);
 	/* If no time format in arg string, assume AM */
 	if (ac < 4) {
@@ -901,9 +937,15 @@ gmsd_init(waypoint *wpt)
 /* usage: xcsv_parse_val("-123.34", *waypt, *field_map)                      */
 /*****************************************************************************/
 static void
-xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp)
+xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp, 
+               route_head **trk)
 {
     char *enclosure = "";
+    geocache_data *gc_data = NULL;
+
+    if (!fmp->printfc) {
+      fatal(MYNAME ": xcsv style '%s' is missing format specifier", fmp->key);
+    }
 
     if (0 == strcmp(fmp->printfc, "\"%s\"")) {
 	enclosure = "\"";
@@ -958,6 +1000,9 @@ xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp)
     case XT_LAT_HUMAN_READABLE:
 	human_to_dec( s, &wpt->latitude, &wpt->longitude, 1 );
 	break;
+    case XT_LAT_DDMMDIR:
+         wpt->latitude = ddmmdir_to_degrees(s);
+      break;
     case XT_LAT_NMEA:
 	wpt->latitude = ddmm2degrees(atof(s));
     	break;
@@ -979,6 +1024,9 @@ xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp)
     case XT_LON_HUMAN_READABLE:
 	human_to_dec( s, &wpt->latitude, &wpt->longitude, 2 );
     	break;
+    case XT_LON_DDMMDIR:
+         wpt->longitude = ddmmdir_to_degrees(s);
+      break;
     case XT_LON_NMEA:
 	wpt->longitude = ddmm2degrees(atof(s));
     	break;
@@ -999,6 +1047,34 @@ xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp)
 	parse_coordinates(s, DATUM_OSGB36, grid_bng,
 		&wpt->latitude, &wpt->longitude, MYNAME);
     	break;
+    case XT_UTM_ZONE:
+        utm_zone = atoi(s);
+        break;
+    case XT_UTM_ZONEC:
+        utm_zonec = atoi(s);
+        break;
+    case XT_UTM_ZONEF:
+        utm_zone = atoi(s);
+        utm_zonec = s[strlen(s) - 1];
+        break;
+    case XT_UTM_EASTING:
+        utm_easting = atof(s);
+        break;
+    case XT_UTM_NORTHING:
+        utm_northing = atof(s);
+        break;
+    case XT_UTM: {
+        char *ss;
+        int i = 0;;
+
+        utm_zone = strtod(s, &ss);
+        utm_zonec = ss[i];
+        ss++;
+        utm_easting = strtod(ss, &ss);
+        while(*ss && !isdigit(*ss)) ss++;
+        utm_northing = strtod(ss, NULL);
+        }
+        break;
     /* ALTITUDE CONVERSIONS ************************************************/
     case XT_ALT_FEET:
 	/* altitude in feet as a decimal value */
@@ -1058,47 +1134,52 @@ xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp)
     case XT_ISO_TIME_MS: 
 	wpt->creation_time = xml_parse_time(s, &wpt->microseconds);
     	break;
-	case XT_GEOCACHE_LAST_FOUND:
-	wpt->gc_data.last_found = yyyymmdd_to_time(s);
+    case XT_NET_TIME: 
+	dotnet_time_to_time_t(atof(s), &wpt->creation_time, &wpt->microseconds);
+	break;
+    case XT_GEOCACHE_LAST_FOUND:
+        waypt_alloc_gc_data(wpt)->last_found = yyyymmdd_to_time(s);
     	break;
 
     /* GEOCACHING STUFF ***************************************************/
     case XT_GEOCACHE_DIFF:
        /* Geocache Difficulty as an int */
-	wpt->gc_data.diff = atof(s) * 10; 
+        waypt_alloc_gc_data(wpt)->diff = atof(s) * 10; 
     	break;
     case XT_GEOCACHE_TERR:
        /* Geocache Terrain as an int */
-	wpt->gc_data.terr = atof(s) * 10;
+        waypt_alloc_gc_data(wpt)->terr = atof(s) * 10;
     	break;
     case XT_GEOCACHE_TYPE:
        /* Geocache Type */
-	wpt->gc_data.type = gs_mktype(s);
+        waypt_alloc_gc_data(wpt)->type = gs_mktype(s);
     	break;
     case XT_GEOCACHE_CONTAINER:
-	wpt->gc_data.container = gs_mkcont(s);
+        waypt_alloc_gc_data(wpt)->container = gs_mkcont(s);
     	break;
     case XT_GEOCACHE_HINT:
-	wpt->gc_data.hint = csv_stringtrim(s, "", 0);
+        waypt_alloc_gc_data(wpt)->hint = csv_stringtrim(s, "", 0);
     	break;
     case XT_GEOCACHE_PLACER:
-	wpt->gc_data.placer = csv_stringtrim(s, "", 0);
+        waypt_alloc_gc_data(wpt)->placer = csv_stringtrim(s, "", 0);
     	break;
     case XT_GEOCACHE_ISAVAILABLE:
+        gc_data = waypt_alloc_gc_data(wpt);
 	if ( case_ignore_strcmp(csv_stringtrim(s, "", 0), "False") == 0 )
-        	wpt->gc_data.is_available = status_false;
+        	gc_data->is_available = status_false;
 	else if ( case_ignore_strcmp(csv_stringtrim(s, "", 0), "True") == 0 )
-		wpt->gc_data.is_available = status_true;
+		gc_data->is_available = status_true;
 	else
-		wpt->gc_data.is_available = status_unknown;
+		gc_data->is_available = status_unknown;
 	break;
     case XT_GEOCACHE_ISARCHIVED:
+        gc_data = waypt_alloc_gc_data(wpt);
 	if ( case_ignore_strcmp(csv_stringtrim(s, "", 0), "False") == 0 )
-		wpt->gc_data.is_archived = status_false;
+		gc_data->is_archived = status_false;
 	else if ( case_ignore_strcmp(csv_stringtrim(s, "", 0), "True") == 0 )
-		wpt->gc_data.is_archived = status_true;        
+		gc_data->is_archived = status_true;        
 	else
-	wpt->gc_data.is_archived = status_unknown;
+	gc_data->is_archived = status_unknown;
 	break;
 		
     /* GPS STUFF *******************************************************/
@@ -1115,7 +1196,7 @@ xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp)
 	wpt->sat = atoi(s);
     	break;
     case XT_GPS_FIX:
-	wpt->fix = atoi(s)-1;
+	wpt->fix = (fix_type)(atoi(s)-(fix_type)1);
 	if ( wpt->fix < fix_2d) {
 	if (!case_ignore_strcmp(s, "none"))
 		wpt->fix = fix_none;
@@ -1131,8 +1212,19 @@ xcsv_parse_val(const char *s, waypoint *wpt, const field_map_t *fmp)
     case XT_ROUTE_NAME:
 	if (csv_route) csv_route->rte_name = csv_stringtrim(s, enclosure, 0);
     	break;
+    case XT_TRACK_NEW: 
+	if (atoi(s) && csv_track && !QUEUE_EMPTY(&csv_track->Q)) {
+		*trk = route_head_alloc();
+		csv_track = *trk;
+
+		track_add_head(*trk);
+	}
+	break;
     case XT_TRACK_NAME:
-	if (csv_track) csv_track->rte_name = csv_stringtrim(s, enclosure, 0);
+	if (!csv_track) {
+		csv_track = route_head_alloc();
+        }
+	csv_track->rte_name = csv_stringtrim(s, enclosure, 0);
     	break;
 	
     /* OTHER STUFF ***************************************************/
@@ -1217,6 +1309,10 @@ xcsv_data_read(void)
     ogue_t *ogp;
     route_head *rte = NULL;
     route_head *trk = NULL;
+    utm_northing = 0;
+    utm_easting = 0;
+    utm_zone = 0;
+    utm_zonec = 'N';
     
     csv_route = csv_track = NULL;
     if (xcsv_file.datatype == trkdata) {
@@ -1227,6 +1323,8 @@ xcsv_data_read(void)
     }
 
     while ((buff = gbfgetstr(xcsv_file.xcsvfp))) {
+	if ((linecount == 0) && xcsv_file.xcsvfp->unicode) cet_convert_init(CET_CHARSET_UTF8, 1);
+
         linecount++;
 	/* Whack trailing space; leading space may matter if our field sep
 	 * is whitespace and we have leading whitespace. 
@@ -1270,7 +1368,7 @@ xcsv_data_read(void)
              */
             while (s) {
                 fmp = (field_map_t *) elem;
-                xcsv_parse_val(s, wpt_tmp, fmp);
+                xcsv_parse_val(s, wpt_tmp, fmp, &trk);
 
                 elem = QUEUE_NEXT(elem);
                 
@@ -1285,11 +1383,21 @@ xcsv_data_read(void)
                 s = csv_lineparse(NULL, xcsv_file.field_delimiter, "",
                   linecount);
             }
+
             if ((xcsv_file.gps_datum > -1) && (xcsv_file.gps_datum != GPS_DATUM_WGS84)) {
         	double alt;
 		GPS_Math_Known_Datum_To_WGS84_M(wpt_tmp->latitude, wpt_tmp->longitude, 0.0,
 		    &wpt_tmp->latitude, &wpt_tmp->longitude, &alt, xcsv_file.gps_datum);
 	    }
+
+            if (utm_easting || utm_northing) {
+                GPS_Math_UTM_EN_To_Known_Datum(&wpt_tmp->latitude,
+                                               &wpt_tmp->longitude,
+                                               utm_easting, utm_northing,
+                                               utm_zone, utm_zonec,
+                                               DATUM_WGS84);
+            }
+
 	    switch(xcsv_file.datatype) {
 		case 0:
 		case wptdata:
@@ -1298,6 +1406,7 @@ xcsv_data_read(void)
 		case trkdata:
 		    if (trk == NULL) {
 			trk = route_head_alloc();
+			csv_track = trk;
 			track_add_head(trk);
 		    }
 		    track_add_wpt(trk, wpt_tmp);
@@ -1305,6 +1414,7 @@ xcsv_data_read(void)
 		case rtedata:
 		    if (rte == NULL) {
 			rte = route_head_alloc();
+			csv_route = rte;
 			route_add_head(rte);
 		    }
 		    route_add_wpt(rte, wpt_tmp);
@@ -1351,6 +1461,9 @@ xcsv_waypt_pr(const waypoint *wpt)
     field_map_t *fmp;
     queue *elem, *tmp;
     double latitude, longitude;
+    int32 utmz;
+    double utme, utmn;
+    char utmzc;
 
     buff[0] = '\0';
     
@@ -1522,6 +1635,10 @@ xcsv_waypt_pr(const waypoint *wpt)
             writebuff(buff, fmp->printfc,
               dec_to_intdeg(lat));
             break;
+        case XT_LAT_DDMMDIR:
+            /*latitude as (degrees * 100) + decimal minutes, with N/S after it */
+            dec_to_human( buff, fmp->printfc, "SN", degrees2ddmm(lat) );
+            break;
 	case XT_LAT_HUMAN_READABLE:
 	    dec_to_human( buff, fmp->printfc, "SN", lat );
 	    break;
@@ -1551,6 +1668,10 @@ xcsv_waypt_pr(const waypoint *wpt)
             writebuff(buff, fmp->printfc,
               dec_to_intdeg(lon));
             break;
+        case XT_LON_DDMMDIR:
+            /* longidute as (degrees * 100) + decimal minutes, with W/E after it*/
+            dec_to_human( buff, fmp->printfc, "WE", degrees2ddmm(lon) );
+            break;
 	case XT_LON_HUMAN_READABLE:
 	    dec_to_human( buff, fmp->printfc, "WE", lon );
 	    break;
@@ -1586,6 +1707,44 @@ xcsv_waypt_pr(const waypoint *wpt)
 		snprintf(buff, sizeof(buff), fmp->printfc, map, (int)(east + 0.5), (int)(north + 0.5));
 		}
 	    break;
+        case XT_UTM: {
+                char tbuf[100];
+                GPS_Math_WGS84_To_UTM_EN(wpt->latitude, wpt->longitude,
+                                         &utme, &utmn, &utmz, &utmzc);
+                snprintf(tbuf, sizeof(tbuf), "%d%c %6.0f %7.0f", 
+                         utmz, utmzc, utme, utmn);
+                writebuff(buff, fmp->printfc, tbuf);
+                }
+                break;
+        case XT_UTM_ZONE:
+                GPS_Math_WGS84_To_UTM_EN(wpt->latitude, wpt->longitude,
+                                         &utme, &utmn, &utmz, &utmzc);
+                writebuff(buff, fmp->printfc, utmz);
+                break;
+        case XT_UTM_ZONEC:
+                GPS_Math_WGS84_To_UTM_EN(wpt->latitude, wpt->longitude,
+                                         &utme, &utmn, &utmz, &utmzc);
+                writebuff(buff, fmp->printfc, utmzc);
+                break;
+        case XT_UTM_ZONEF: {
+                char tbuf[10];
+                GPS_Math_WGS84_To_UTM_EN(wpt->latitude, wpt->longitude,
+                                         &utme, &utmn, &utmz, &utmzc);
+                tbuf[0] = 0;
+                snprintf(tbuf, sizeof(tbuf), "%d%c", utmz, utmzc);
+                writebuff(buff, fmp->printfc, tbuf);
+                }
+                break;
+        case XT_UTM_NORTHING:
+                GPS_Math_WGS84_To_UTM_EN(wpt->latitude, wpt->longitude,
+                                         &utme, &utmn, &utmz, &utmzc);
+                writebuff(buff, fmp->printfc, utmn);
+                break;
+        case XT_UTM_EASTING:
+                GPS_Math_WGS84_To_UTM_EN(wpt->latitude, wpt->longitude,
+                                         &utme, &utmn, &utmz, &utmzc);
+                writebuff(buff, fmp->printfc, utme);
+                break;
 
         /* ALTITUDE CONVERSIONS**********************************************/
         case XT_ALT_FEET:
@@ -1664,50 +1823,50 @@ xcsv_waypt_pr(const waypoint *wpt)
 		wpt->microseconds, XML_LONG_TIME);
 	    break;
         case XT_GEOCACHE_LAST_FOUND:
-	    writebuff(buff, fmp->printfc, time_to_yyyymmdd(wpt->gc_data.last_found));
+	    writebuff(buff, fmp->printfc, time_to_yyyymmdd(wpt->gc_data->last_found));
 	    break;
 
         /* GEOCACHE STUFF **************************************************/
         case XT_GEOCACHE_DIFF:
             /* Geocache Difficulty as a double */
-            writebuff(buff, fmp->printfc, wpt->gc_data.diff / 10.0);
-	    field_is_unknown = !wpt->gc_data.diff;
+            writebuff(buff, fmp->printfc, wpt->gc_data->diff / 10.0);
+	    field_is_unknown = !wpt->gc_data->diff;
             break;
         case XT_GEOCACHE_TERR:
             /* Geocache Terrain as a double */
-            writebuff(buff, fmp->printfc, wpt->gc_data.terr / 10.0);
-	    field_is_unknown = !wpt->gc_data.terr;
+            writebuff(buff, fmp->printfc, wpt->gc_data->terr / 10.0);
+	    field_is_unknown = !wpt->gc_data->terr;
             break;
         case XT_GEOCACHE_CONTAINER:
             /* Geocache Container */
-            writebuff(buff, fmp->printfc, gs_get_container(wpt->gc_data.container));
-	    field_is_unknown = wpt->gc_data.container == gc_unknown;
+            writebuff(buff, fmp->printfc, gs_get_container(wpt->gc_data->container));
+	    field_is_unknown = wpt->gc_data->container == gc_unknown;
 	    break;
 	case XT_GEOCACHE_TYPE:
             /* Geocache Type */
-            writebuff(buff, fmp->printfc, gs_get_cachetype(wpt->gc_data.type));
-	    field_is_unknown = wpt->gc_data.type == gt_unknown;
+            writebuff(buff, fmp->printfc, gs_get_cachetype(wpt->gc_data->type));
+	    field_is_unknown = wpt->gc_data->type == gt_unknown;
             break; 
 	case XT_GEOCACHE_HINT:
-	    writebuff(buff, fmp->printfc, NONULL(wpt->gc_data.hint));
-	    field_is_unknown = !wpt->gc_data.hint;
+	    writebuff(buff, fmp->printfc, NONULL(wpt->gc_data->hint));
+	    field_is_unknown = !wpt->gc_data->hint;
             break; 
 	case XT_GEOCACHE_PLACER:
-	    writebuff(buff, fmp->printfc, NONULL(wpt->gc_data.placer));
-	    field_is_unknown = !wpt->gc_data.placer;
+	    writebuff(buff, fmp->printfc, NONULL(wpt->gc_data->placer));
+	    field_is_unknown = !wpt->gc_data->placer;
             break;
 	case XT_GEOCACHE_ISAVAILABLE:
-	    if ( wpt->gc_data.is_available == status_false )
+	    if ( wpt->gc_data->is_available == status_false )
 	      writebuff(buff, fmp->printfc, "False");
-	    else if ( wpt->gc_data.is_available == status_true )
+	    else if ( wpt->gc_data->is_available == status_true )
 	      writebuff(buff, fmp->printfc, "True");
 			else
 				writebuff(buff, fmp->printfc, "Unknown");
             break;
 	case XT_GEOCACHE_ISARCHIVED:
-	    if ( wpt->gc_data.is_archived == status_false )
+	    if ( wpt->gc_data->is_archived == status_false )
 	      writebuff(buff, fmp->printfc, "False");
-	    else if ( wpt->gc_data.is_archived == status_true )
+	    else if ( wpt->gc_data->is_archived == status_true )
 	    	writebuff(buff, fmp->printfc, "True");
 	    else
 		writebuff(buff, fmp->printfc, "Unknown");
@@ -1799,6 +1958,13 @@ xcsv_waypt_pr(const waypoint *wpt)
 		writebuff(buff, fmp->printfc, GMSD_GET(facility, ""));
 		}
 		break;
+	/* specials */
+	case XT_FILENAME:
+		writebuff(buff, fmp->printfc, wpt->session->filename);
+		break;
+	case XT_FORMAT:
+		writebuff(buff, fmp->printfc, wpt->session->name);
+		break;
 	case -1:
 		if (strncmp(fmp->key, "LON_10E", 7) == 0) {
 			writebuff(buff, fmp->printfc, lon * pow((double)10, atof(fmp->key+7)));
