@@ -25,7 +25,9 @@
 #include <string.h>
 #include <stdarg.h>
 #include <stddef.h>
+#if HAVE_CONFIG_H
 #include "config.h"
+#endif
 #include "queue.h"
 #include "gbtypes.h"
 #if HAVE_LIBZ
@@ -37,6 +39,7 @@
 #include "cet.h"
 #include "cet_util.h"
 #include "inifile.h"
+#include "session.h"
 
 // Turn on Unicode in expat?
 #ifdef _UNICODE
@@ -50,6 +53,14 @@
 #  define M_PI 3.14159265358979323846
 #endif
 
+#ifndef FALSE
+#  define FALSE 0
+#endif
+
+#ifndef TRUE
+#  define TRUE !FALSE
+#endif
+
 #define FEET_TO_METERS(feetsies) ((feetsies) * 0.3048)
 #define METERS_TO_FEET(meetsies) ((meetsies) * 3.2808399)
 
@@ -101,8 +112,10 @@
 #  if _MSC_VER
 #    pragma warning(disable:4244)
 #  endif
+#if !defined _CRT_SECURE_NO_DEPRECATE
 #  define _CRT_SECURE_NO_DEPRECATE 1
 #endif
+#endif
 
 /* Pathname separator character */
 #if __WIN32__
@@ -228,7 +241,8 @@ typedef enum {
 	gt_benchmark, /* Extension to Groundspeak for GSAK */
 	gt_cito,
 	gt_ape,
-	gt_mega
+	gt_mega,
+	gt_wherigo
 } geocache_type;
 
 typedef enum {
@@ -288,6 +302,12 @@ typedef struct format_specific_data {
 	fs_convert convert;
 } format_specific_data;
 
+typedef struct {
+  int bbggrr;   // 32 bit color: Blue/Green/Red.  < 0 == unknown.
+  unsigned char opacity;  // 0 == transparent.  255 == opaque.
+} gb_color;
+
+
 format_specific_data *fs_chain_copy( format_specific_data *source );
 void fs_chain_destroy( format_specific_data *chain );
 format_specific_data *fs_chain_find( format_specific_data *chain, long type );
@@ -334,9 +354,25 @@ typedef struct {
 	unsigned int altitude:1;		/+ altitude field is set +/
 	... and others
 	*/
+	unsigned int is_split:1;		/* the waypoint represents a split */
+	unsigned int new_trkseg:1;		/* True if first in new trkseg. */
+
 	
 } wp_flags;
 
+// These are dicey as they're collected on read. Subsequent filters may change
+// things, though it's u nlikely to matter in practical terms.  Don't use these
+// if a false positive would be deleterious.
+typedef struct {
+  unsigned int trait_geocaches:1;
+  unsigned int trait_heartrate:1;
+  unsigned int trait_cadence:1;
+  unsigned int trait_power:1;
+  unsigned int trait_depth:1;
+  unsigned int trait_temperature:1;
+} global_trait;
+const global_trait* get_traits();
+
 #define WAYPT_SET(wpt,member,val) { wpt->member = (val); wpt->wpt_flags.member = 1; }
 #define WAYPT_GET(wpt,member,def) ((wpt->wpt_flags.member) ? (wpt->member) : (def))
 #define WAYPT_UNSET(wpt,member) wpt->wpt_flags.member = 0
@@ -432,9 +468,11 @@ typedef struct {
 
 	unsigned char heartrate; /* Beats/min. likely to get moved to fs. */
 	unsigned char cadence;	 /* revolutions per minute */
+	float power; /* watts, as measured by cyclists */
 	float temperature; /* Degrees celsius */
-	geocache_data gc_data;
+	const geocache_data *gc_data;
 	format_specific_data *fs;
+	session_t *session;	/* pointer to a session struct */
 	void *extra_data;	/* Extra data added by, say, a filter. */
 } waypoint;
 
@@ -448,6 +486,9 @@ typedef struct {
 	int rte_waypt_ct;		/* # waypoints in waypoint list */
 	format_specific_data *fs;
 	unsigned short cet_converted;	/* strings are converted to UTF8; interesting only for input */
+        gb_color line_color;         /* Optional line color for rendering */
+        int line_width;         /* in pixels (sigh).  < 0 is unknown. */
+	session_t *session;	/* pointer to a session struct */
 } route_head;
 
 /*
@@ -516,10 +557,13 @@ waypoint * waypt_new(void);
 void waypt_del (waypoint *);
 void waypt_free (waypoint *);
 void waypt_disp_all(waypt_cb);
+void waypt_disp_session(const session_t *se, waypt_cb cb);
 void waypt_init_bounds(bounds *bounds);
 int waypt_bounds_valid(bounds *bounds);
 void waypt_add_to_bounds(bounds *bounds, const waypoint *waypointp);
 void waypt_compute_bounds(bounds *);
+double gcgeodist(const double lat1, const double lon1,
+                 const double lat2, const double lon2);
 void waypt_flush(queue *);
 void waypt_flush_all(void);
 unsigned int waypt_count(void);
@@ -532,6 +576,11 @@ waypoint * find_waypt_by_name(const char *name);
 void waypt_backup(signed int *count, queue **head_bak);
 void waypt_restore(signed int count, queue *head_bak);
 
+geocache_data *waypt_alloc_gc_data(waypoint *wpt);
+int waypt_empty_gc_data(const waypoint *wpt);
+geocache_type gs_mktype(const char *t);
+geocache_container gs_mkcont(const char *t);
+
 route_head *route_head_alloc(void);
 void route_add (waypoint *);
 void route_add_wpt(route_head *rte, waypoint *wpt);
@@ -544,9 +593,12 @@ void route_reverse(const route_head *rte_hd);
 waypoint * route_find_waypt_by_name(route_head *rh, const char *name);
 void track_add_head(route_head *rte);
 void track_del_head(route_head *rte);
+void track_insert_head(route_head *rte, route_head *predecessor);
 void route_disp(const route_head *rte, waypt_cb);
 void route_disp_all(route_hdr, route_trl, waypt_cb);
 void track_disp_all(route_hdr, route_trl, waypt_cb);
+void route_disp_session(const session_t *se, route_hdr rh, route_trl rt, waypt_cb wc);
+void track_disp_session(const session_t *se, route_hdr rh, route_trl rt, waypt_cb wc);
 void route_flush( queue *);
 void route_flush_all(void);
 void route_flush_all_routes(void);
@@ -591,6 +643,7 @@ void setshort_mustuniq(short_handle,  int n);
 void setshort_whitespace_ok(short_handle,  int n);
 void setshort_repeating_whitespace_ok(short_handle,  int n);
 void setshort_defname(short_handle, const char *s);
+void setshort_is_utf8(short_handle h, const int is_utf8);
 
 /*
  *  Vmem flags values.
@@ -643,13 +696,14 @@ void 	vmem_realloc(vmem_t*, size_t);
 #define ARG_TERMINATOR {0, 0, 0, 0, 0, ARG_NOMINMAX}
 
 typedef struct arglist {
-	char *argstring;
+	const char *argstring;
 	char **argval;
-	char *helpstring;
-	char *defaultvalue;
-	gbuint32 argtype;
-	char *minvalue;		/* minimum value for numeric options */
-	char *maxvalue;		/* maximum value for numeric options */
+	const char *helpstring;
+	const char *defaultvalue;
+	const gbuint32 argtype;
+	const char *minvalue;		/* minimum value for numeric options */
+	const char *maxvalue;		/* maximum value for numeric options */
+	char *argvalptr;	/* !!! internal helper. Not used in definitions !!! */
 } arglist_t;
 
 typedef enum {
@@ -703,9 +757,10 @@ typedef struct ff_vecs {
 	ff_write write;
 	ff_exit exit;
 	arglist_t *args;
-	char *encode;
+	const char *encode;
 	int fixed_encode;
 	position_ops_t position_ops;
+	const char *name;		/* dyn. initialized by find_vec */
 } ff_vecs_t;
 
 typedef struct style_vecs {
@@ -734,6 +789,7 @@ void assign_option(const char *vecname, arglist_t *ap, const char *val);
 void disp_vec_options(const char *vecname, arglist_t *ap);
 void disp_vecs(void);
 void disp_vec( const char *vecname );
+void init_vecs(void);
 void exit_vecs(void);
 void disp_formats(int version);
 const char * name_option(long type);
@@ -806,6 +862,7 @@ signed int get_tz_offset(void);
 time_t mklocaltime(struct tm *t);
 time_t mkgmtime(struct tm *t);
 time_t current_time(void);
+void dotnet_time_to_time_t(double dotnet, time_t *t, int *ms);
 signed int month_lookup(const char *m);
 const char *get_cache_icon(const waypoint *waypointp);
 const char *gs_get_cachetype(geocache_type t);
@@ -816,7 +873,7 @@ char * strip_html(const utf_string*);
 char * strip_nastyhtml(const char * in);
 char * convert_human_date_format(const char *human_datef);	/* "MM,YYYY,DD" -> "%m,%Y,%d" */
 char * convert_human_time_format(const char *human_timef);	/* "HH+mm+ss"   -> "%H+%M+%S" */
-char * pretty_deg_format(double lat, double lon, char fmt, char *sep, int html);   /* decimal ->  dd.dddd or dd mm.mmm or dd mm ss */
+char * pretty_deg_format(double lat, double lon, char fmt, const char *sep, int html);   /* decimal ->  dd.dddd or dd mm.mmm or dd mm ss */
 
 char * get_filename(const char *fname);				/* extract the filename portion */
 
@@ -827,6 +884,7 @@ char * get_filename(const char *fname);				/* extract the filename portion */
 #define CET_NOT_CONVERTABLE_DEFAULT '$'
 #define CET_CHARSET_ASCII	"US-ASCII"
 #define CET_CHARSET_UTF8	"UTF-8"
+#define CET_CHARSET_HEBREW  "CP1255"
 #define CET_CHARSET_MS_ANSI	"MS-ANSI"
 #define CET_CHARSET_LATIN1	"ISO-8859-1"
 
@@ -839,9 +897,9 @@ char * get_filename(const char *fname);				/* extract the filename portion */
 /* this lives in gpx.c */
 time_t xml_parse_time( const char *cdatastr, int * microsecs );
 	
-xml_tag *xml_findfirst( xml_tag *root, char *tagname );
-xml_tag *xml_findnext( xml_tag *root, xml_tag *cur, char *tagname );
-char *xml_attribute( xml_tag *tag, char *attrname );
+xml_tag *xml_findfirst( xml_tag *root, const char *tagname );
+xml_tag *xml_findnext( xml_tag *root, xml_tag *cur, const char *tagname );
+char *xml_attribute( xml_tag *tag, const char *attrname );
 
 char * rot13( const char *str );
 
@@ -876,6 +934,7 @@ typedef struct {
  */
 
 signed int be_read16(const void *p);
+unsigned int be_readu16(const void *p);
 signed int be_read32(const void *p);
 signed int le_read16(const void *p);
 unsigned int le_readu16(const void *p);
@@ -887,8 +946,8 @@ void be_write32(void *pp, const unsigned i);
 void le_write16(void *pp, const unsigned i);
 void le_write32(void *pp, const unsigned i);
 
-double endian_read_double(void* ptr, int read_le);
-float  endian_read_float(void* ptr, int read_le);
+double endian_read_double(const void* ptr, int read_le);
+float  endian_read_float(const void* ptr, int read_le);
 void   endian_write_double(void* ptr, double d, int write_le);
 void   endian_write_float(void* ptr, float f, int write_le);
 
@@ -897,8 +956,8 @@ double be_read_double(void *p);
 void   be_write_float(void *pp, float d);
 void   be_write_double(void *pp, double d);
 
-float  le_read_float(void *p);
-double le_read_double(void *p);
+float  le_read_float(const void *p);
+double le_read_double(const void *p);
 void   le_write_float(void *ptr, float f);
 void   le_write_double(void *p, double d);
 
@@ -930,6 +989,14 @@ typedef enum {
 #define DATUM_OSGB36	86
 #define DATUM_WGS84	118
 
+/* bit manipulation functions (util.c) */
+
+char gb_getbit(const void *buf, const gbuint32 nr);
+void gb_setbit(void *buf, const gbuint32 nr);
+
+void *gb_int2ptr(const int i);
+int gb_ptr2int(const void *p);
+
 /*
  *  From parse.c
  */
@@ -937,6 +1004,7 @@ int parse_coordinates(const char *str, int datum, const grid_type grid,
 	double *latitude, double *longitude, const char *module);
 int parse_distance(const char *str, double *val, double scale, const char *module);
 int parse_speed(const char *str, double *val, const double scale, const char *module);
+time_t parse_date(const char *str, const char *format, const char *module);
 
 /*
  *  From util_crc.c
@@ -970,12 +1038,13 @@ int nmea_cksum(const char *const buf);
 /*
  * Color helpers.
  */
-int color_to_bbggrr(char *cname);
+int color_to_bbggrr(const char *cname);
 
 /*
  * A constant for unknown altitude.   It's tempting to just use zero
  * but that's not very nice for the folks near sea level.
  */
 #define unknown_alt 	-99999999.0
+#define unknown_color	-1
 
 #endif /* gpsbabel_defs_h_included */
