@@ -131,6 +131,12 @@ fl_vecs_t filter_vec_list[] = {
 		"Swap latitude and longitude of all loaded points"
 	},
 	
+#elif defined (MINIMAL_FILTERS)
+	{
+		&trackfilter_vecs,
+		"track",
+		"Manipulate track lists"
+	},
 #endif
         {
 		NULL,
@@ -289,8 +295,8 @@ disp_filter_vec( const char *vecname )
 static signed int
 alpha (const void *a, const void *b)
 {
-        const fl_vecs_t *const ap = a;
-        const fl_vecs_t *const bp = b;
+        const fl_vecs_t *const ap = (const fl_vecs_t*) a;
+        const fl_vecs_t *const bp = (const fl_vecs_t*) b;
 
         return case_ignore_strcmp(ap->desc , bp->desc);
 }
