@@ -110,8 +110,8 @@
 
 /*******************************************************************************/
 
-/* static char gdb_release[] = "$Revision: 1.67 $"; */
-static char gdb_release_date[] = "$Date: 2008/09/11 20:41:43 $";
+/* static char gdb_release[] = "$Revision: 1.72 $"; */
+static char gdb_release_date[] = "$Date: 2010/04/11 22:41:17 $";
 
 static gbfile *fin, *fout, *ftmp;
 static int gdb_ver, gdb_category, gdb_via, gdb_roadbook;
@@ -432,7 +432,7 @@ read_file_header(void)
 	}
 
 	i = FREAD_STR(buf);
-	is_fatal((i != 9) || (strcmp(buf, "MapSource") != 0), "Invalid header!");
+	is_fatal(!(((i == 9) && (strcmp(buf, "MapSource") == 0)) || ((i == 8) && (strcmp(buf, "BaseCamp") == 0))), MYNAME ": Not a recognized signature in header");
 }
 
 /*-----------------------------------------------------------------------------*/
@@ -441,7 +441,8 @@ static waypoint *
 read_waypoint(gt_waypt_classes_e *waypt_class_out)
 {
 	char buf[128];		/* used for temporary stuff */
-	int wpt_class, display, icon, dynamic;
+	int display, icon, dynamic;
+	gt_waypt_classes_e wpt_class;
 	int i;
 	waypoint *res;
 	garmin_fs_t *gmsd;
@@ -994,7 +995,9 @@ read_data(void)
 		route_head *trk, *rte;
 		
 		len = FREAD_i32;
-		FREAD(&typ, 1);
+		if (FREAD(&typ, 1) < 1) {
+                  fatal(MYNAME ": Attempt to read past EOF.");
+                }
 		if (typ == 'V') break;	/* break the loop */
 
 		gbfrewind(ftmp);
@@ -1035,6 +1038,11 @@ read_data(void)
 		
 		fin = fsave;
 		delta = len - gbftell(ftmp);
+                is_fatal(delta > 1000000, "Internal consistency error.  Delta too big");
+
+                // Avoid finite loop on bogus beta files from '06.
+                // THe 100000 is totally pulled from my hat.
+                // is_fatal((delta > 1000000) || (delta < 0), "Internal GDB error; invalid delta.");
 
 		if (dump && delta) {
 			if (! incomplete++) {
@@ -1049,7 +1057,8 @@ read_data(void)
 			if (delta > 0) {
 				int i;
 				char *buf = xmalloc(delta);
-				FREAD(buf, delta);
+				if (FREAD(buf, delta) < 1)
+                                  fatal(MYNAME ": Attempt to read past EOF.\n");
 				for (i = 0; i < delta; i++) {
 					warning(" %02x", (unsigned char)buf[i]);
 				}
@@ -1249,7 +1258,7 @@ write_waypoint(
 	else /* if (gdb_ver > GDB_VER_3) */ {
 		int cnt;
 		url_link *url_next;
-		char *str;
+		const char *str;
 
 		if (wpt_class < gt_waypt_class_map_point)	/* street address */
 			str  = GMSD_GET(addr, "");
@@ -1282,7 +1291,7 @@ write_waypoint(
 
 	/* VERSION DEPENDENT CODE */
 	if (gdb_ver >= GDB_VER_3) {
-		char *str = GMSD_GET(phone_nr, "");
+		const char *str = GMSD_GET(phone_nr, "");
 		if (*str) {
 			FWRITE_i32(1);
 			FWRITE_CSTR(str);
