@@ -1,7 +1,8 @@
 /*
     Access GPX data files.
 
-    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008 Robert Lipe, robertlipe@usa.net
+    Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008,
+      2009, 2010 Robert Lipe, gpsbabel.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -64,8 +65,10 @@ static char *suppresswhite = NULL;
 static char *urlbase = NULL;
 static route_head *trk_head;
 static route_head *rte_head;
+static const route_head *current_trk_head;		// Output.
 /* used for bounds calculation on output */
 static bounds all_bounds;
+static int next_trkpt_is_new_seg;
 
 static format_specific_data **fs_ptr;
 
@@ -178,6 +181,8 @@ typedef enum {
 	tt_trk_trkseg_trkpt_time,
 	tt_trk_trkseg_trkpt_course,	
 	tt_trk_trkseg_trkpt_speed,
+        tt_trk_trkseg_trkpt_heartrate,
+        tt_trk_trkseg_trkpt_cadence,
 
 	tt_humminbird_wpt_depth,
 	tt_humminbird_wpt_status,
@@ -302,6 +307,8 @@ tag_mapping tag_path_map[] = {
   {type, 1, "/gpx/wpt/geocache/" name, 0UL }  /* opencaching.de */
 
 #define GARMIN_WPT_EXT "/gpx/wpt/extensions/gpxx:WaypointExtension"
+#define GARMIN_TRK_EXT "/gpx/trk/trkseg/trkpt/extensions/gpxtpx:TrackPointExtension"
+#define GARMIN_RTE_EXT "/gpx/rte/rtept/extensions/gpxxx:RoutePointExtension"
 
 //	GEOTAG( tt_cache, 		"cache"),
 	{ tt_cache, 1, "/gpx/wpt/groundspeak:cache" },
@@ -328,6 +335,7 @@ tag_mapping tag_path_map[] = {
 	{ tt_garmin_wpt_extensions, 0, GARMIN_WPT_EXT, 0UL },
 	{ tt_garmin_wpt_proximity, 0, GARMIN_WPT_EXT "/gpxx:Proximity", 0UL },
 	{ tt_garmin_wpt_temperature, 0, GARMIN_WPT_EXT "/gpxx:Temperature", 0UL },
+	{ tt_garmin_wpt_temperature, 0, GARMIN_TRK_EXT "/gpxtpx:atemp", 0UL },
 	{ tt_garmin_wpt_depth, 0, GARMIN_WPT_EXT "/gpxx:Depth", 0UL },
 	{ tt_garmin_wpt_display_mode, 0, GARMIN_WPT_EXT "/gpxx:DisplayMode", 0UL },
 	{ tt_garmin_wpt_categories, 0, GARMIN_WPT_EXT "/gpxx:Categories", 0UL },
@@ -339,6 +347,10 @@ tag_mapping tag_path_map[] = {
 	{ tt_garmin_wpt_postal_code, 0, GARMIN_WPT_EXT "/gpxx:Address/gpxx:PostalCode", 0UL },
 	{ tt_garmin_wpt_phone_nr, 0, GARMIN_WPT_EXT "/gpxx:PhoneNumber", 0UL },
 
+        // In Garmin space, but in core of waypoint.
+	{ tt_trk_trkseg_trkpt_heartrate, 0, GARMIN_TRK_EXT "/gpxtpx:hr", 0UL },
+	{ tt_trk_trkseg_trkpt_cadence, 0, GARMIN_TRK_EXT "/gpxtpx:cad", 0UL },
+
 	{ tt_humminbird_wpt_depth, 0, "/gpx/wpt/extensions/h:depth", 0UL },  // in centimeters.
 	{ tt_humminbird_wpt_status, 0, "/gpx/wpt/extensions/h:status", 0UL },
 
@@ -438,9 +450,10 @@ tag_gpx(const char **attrv)
 		 */
 		else if (strcmp(avp[0], "xsi:schemaLocation") == 0) {
 			if (0 == strcmp(avp[1], DEFAULT_XSI_SCHEMA_LOC_11)) {
-				if (0 == strcmp(xsi_schema_loc, DEFAULT_XSI_SCHEMA_LOC))
+				if (0 == strcmp(xsi_schema_loc, DEFAULT_XSI_SCHEMA_LOC)) {
 					xfree(xsi_schema_loc);
 					xsi_schema_loc = xstrdup(DEFAULT_XSI_SCHEMA_LOC_11);
+				}
 				continue;
 			}
 			if (0 == strstr(xsi_schema_loc, avp[1])) {
@@ -682,6 +695,10 @@ gpx_start(void *data, const XML_Char *xml_el, const XML_Char **xml_attr)
 		break;
 	case tt_trk_trkseg_trkpt:
 		tag_wpt(attr);
+		if (next_trkpt_is_new_seg) {
+			wpt_tmp->wpt_flags.new_trkseg = 1;
+			next_trkpt_is_new_seg = 0;
+		}
 		break;
 	case tt_unknown:
 		start_something_else(el, attr);
@@ -734,6 +751,7 @@ gs_type_mapping{
 	{ gt_locationless, "Locationless (Reverse) Cache" },
 	{ gt_ape, "Project APE Cache" },
 	{ gt_mega, "Mega-Event Cache" },
+	{ gt_wherigo, "Wherigo Cache" },
 
 	{ gt_benchmark, "Benchmark" }, /* Not Groundspeak; for GSAK  */
 };
@@ -1039,6 +1057,9 @@ gpx_end(void *data, const XML_Char *xml_el)
 		break;
 	case tt_trk:
 		break;
+	case tt_trk_trkseg:
+		next_trkpt_is_new_seg = 1;
+		break;
 	case tt_trk_trkseg_trkpt:
 		track_add_wpt(trk_head, wpt_tmp);
 		wpt_tmp = NULL;
@@ -1055,6 +1076,12 @@ gpx_end(void *data, const XML_Char *xml_el)
 	case tt_trk_trkseg_trkpt_speed:
 		WAYPT_SET(wpt_tmp, speed, atof(cdatastrp));
 		break;
+        case tt_trk_trkseg_trkpt_heartrate:
+                wpt_tmp->heartrate = atof(cdatastrp);
+                break;
+        case tt_trk_trkseg_trkpt_cadence:
+                wpt_tmp->cadence = atof(cdatastrp);
+                break;
 
 	/*
 	 * Items that are actually in multiple categories.
@@ -1591,19 +1618,38 @@ gpx_write_common_position(const waypoint *waypointp, const char *indent)
 }
 
 static void
-gpx_write_common_depth(const waypoint *waypointp, const char *indent)
+gpx_write_common_extensions(const waypoint *waypointp, const char *indent)
 {
-	if (waypointp->depth != 0) {
-		if (opt_humminbirdext || opt_garminext) {
-			gbfprintf(ofd, "%s<extensions>\n", indent);
+	if (((opt_humminbirdext || opt_garminext) && (waypointp->depth != 0 || waypointp->temperature != 0))
+	    || (opt_garminext && (waypointp->heartrate != 0 || waypointp->cadence != 0))) {
+		gbfprintf(ofd, "%s<extensions>\n", indent);
+		if (waypointp->depth != 0) {
 			if (opt_humminbirdext)
 				gbfprintf(ofd, "%s  <h:depth>%f</h:depth>\n",
-				          indent, waypointp->depth*100.0);
+			        	  indent, waypointp->depth*100.0);
 			if (opt_garminext)
 				gbfprintf(ofd, "%s  <gpxx:Depth>%f</gpxx:Depth>\n",
-				          indent, waypointp->depth);
-			gbfprintf(ofd, "%s</extensions>\n", indent);
+			        	  indent, waypointp->depth);
 		}
+		if (waypointp->temperature != 0) {
+			if (opt_humminbirdext)
+				gbfprintf(ofd, "%s  <h:temperature>%f</h:temperature>\n",
+			        	  indent, waypointp->temperature);
+			if (opt_garminext)
+				gbfprintf(ofd, "%s  <gpxx:Temperature>%f</gpxx:Temperature>\n",
+				          indent, waypointp->temperature);
+		}
+		if (opt_garminext && (waypointp->heartrate != 0 || waypointp->cadence != 0)) {
+			gbfprintf(ofd, "%s  <gpxtpx:TrackPointExtension>\n", indent);
+			if (waypointp->heartrate != 0)
+				gbfprintf(ofd, "%s    <gpxtpx:hr>%u</gpxtpx:hr>\n",
+				          indent, waypointp->heartrate);
+			if (waypointp->cadence != 0)
+				gbfprintf(ofd, "%s    <gpxtpx:cad>%u</gpxtpx:cad>\n",
+				          indent, waypointp->cadence);
+			gbfprintf(ofd, "%s  </gpxtpx:TrackPointExtension>\n", indent);
+		}
+		gbfprintf(ofd, "%s</extensions>\n", indent);
 	}
 }
 
@@ -1661,7 +1707,7 @@ gpx_waypt_pr(const waypoint *waypointp)
 		/* MapSource doesn't accepts extensions from 1.0 */
 		garmin_fs_xml_fprint(ofd, waypointp);
 	}
-	gpx_write_common_depth(waypointp, "  ");
+	gpx_write_common_extensions(waypointp, "  ");
 	gbfprintf(ofd, "</wpt>\n");
 }
 
@@ -1669,6 +1715,7 @@ static void
 gpx_track_hdr(const route_head *rte)
 {
 	fs_xml *fs_gpx;
+	current_trk_head = rte;
 
 	gbfprintf(ofd, "<trk>\n");
 	write_optional_xml_entity(ofd, "  ", "name", rte->rte_name);
@@ -1684,13 +1731,21 @@ gpx_track_hdr(const route_head *rte)
 		}
 	}
 
-	gbfprintf(ofd, "<trkseg>\n");
 }
 
 static void
 gpx_track_disp(const waypoint *waypointp)
 {
 	fs_xml *fs_gpx;
+	int first_in_trk;
+	first_in_trk = waypointp->Q.prev == &current_trk_head->waypoint_list;
+
+	if (waypointp->wpt_flags.new_trkseg) {
+		if (!first_in_trk) {
+			gbfprintf(ofd, "</trkseg>\n");
+		}
+		gbfprintf(ofd, "<trkseg>\n");
+	}
 
 	gbfprintf(ofd, "<trkpt lat=\"" FLT_FMT_T "\" lon=\"" FLT_FMT_T "\">\n",
 		waypointp->latitude,
@@ -1723,15 +1778,18 @@ gpx_track_disp(const waypoint *waypointp)
 		fprint_xml_chain( fs_gpx->tag, waypointp );
 	}
 
-	gpx_write_common_depth(waypointp, "  ");
+	gpx_write_common_extensions(waypointp, "  ");
 	gbfprintf(ofd, "</trkpt>\n");
 }
 
 static void
 gpx_track_tlr(const route_head *rte)
 {
-	gbfprintf(ofd, "</trkseg>\n");
+	if (!QUEUE_EMPTY(&current_trk_head->waypoint_list)) {
+		gbfprintf(ofd, "</trkseg>\n");
+	}
 	gbfprintf(ofd, "</trk>\n");
+	current_trk_head = NULL;
 }
 
 static
@@ -1778,7 +1836,7 @@ gpx_route_disp(const waypoint *waypointp)
 		fprint_xml_chain( fs_gpx->tag, waypointp );
 	}
 
-	gpx_write_common_depth(waypointp, "    ");
+	gpx_write_common_extensions(waypointp, "    ");
 	gbfprintf(ofd, "  </rtept>\n");
 }
 
@@ -1851,7 +1909,8 @@ gpx_write(void)
 	if (opt_humminbirdext)
 		gbfprintf(ofd, "  xmlns:h=\"http://humminbird.com\"\n");
 	if (opt_garminext)
-		gbfprintf(ofd, "  xmlns:gpxx=\"http://www.garmin.com/xmlschemas/GpxExtensions/v3\"");
+		gbfprintf(ofd, "  xmlns:gpxx=\"http://www.garmin.com/xmlschemas/GpxExtensions/v3\"\n"
+		               "  xmlns:gpxtpx=\"http://www.garmin.com/xmlschemas/TrackPointExtension/v1\"\n");
 
 	gbfprintf(ofd, "  xmlns=\"http://www.topografix.com/GPX/%c/%c\"\n", gpx_wversion[0], gpx_wversion[2]);
 	if (xsi_schema_loc) {
