@@ -22,13 +22,15 @@
 
 #include <stdio.h>
 #include <ctype.h>
+#if HAVE_CONFIG_H
 #include "config.h"
+#endif
 #if HAVE_LIBUSB
 #include <usb.h>
 #include "gps.h"
 #include "garminusb.h"
 #include "gpsusbcommon.h"
-#include "garmin_device_xml.h"
+#include "../garmin_device_xml.h"
 
 #define GARMIN_VID 0x91e
 
@@ -41,8 +43,8 @@
  * coalescion into packets anyway becuase of their serial background) will
  * compensate.
  */
-#define TMOUT_I 3000 /*  Milliseconds to timeout intr pipe access. */
-#define TMOUT_B 3000 /*  Milliseconds to timeout bulk pipe access. */
+#define TMOUT_I 5000 /*  Milliseconds to timeout intr pipe access. */
+#define TMOUT_B 5000 /*  Milliseconds to timeout bulk pipe access. */
 
 typedef struct {
 	struct usb_bus *busses;
@@ -269,6 +271,16 @@ garmin_usb_start(struct usb_device *dev, libusb_unit_data *lud)
 	/*
 	 * Hrmph.  No iManufacturer or iProduct headers....
 	 */
+
+
+#if __APPLE__
+	// On Leopard, if we don't do an explicit set_configuration, some
+	// devices will work only the first time after a reset.
+	if (usb_set_configuration(udev, 1) < 0) {
+		fatal("usb_set_configuration failed: %s\n", usb_strerror());
+	};
+#endif
+
 #if 0
 	if (usb_set_configuration(udev, 1) < 0) {
 #if __linux__
@@ -316,13 +328,13 @@ garmin_usb_start(struct usb_device *dev, libusb_unit_data *lud)
 #define EA(x) x & USB_ENDPOINT_ADDRESS_MASK
 			case USB_ENDPOINT_TYPE_BULK:
 				if (ep->bEndpointAddress & USB_ENDPOINT_DIR_MASK)
-					gusb_bulk_in_ep = EA(ep->bEndpointAddress);
+					gusb_bulk_in_ep = (EA(ep->bEndpointAddress)) | USB_ENDPOINT_IN;
 				else
 					gusb_bulk_out_ep = EA(ep->bEndpointAddress);
 				break;
 			case USB_ENDPOINT_TYPE_INTERRUPT:
 				if (ep->bEndpointAddress & USB_ENDPOINT_DIR_MASK)
-					gusb_intr_in_ep = EA(ep->bEndpointAddress);
+					gusb_intr_in_ep = (EA(ep->bEndpointAddress)) | USB_ENDPOINT_IN;
 				break;
 		}
 	}
@@ -338,7 +350,12 @@ garmin_usb_start(struct usb_device *dev, libusb_unit_data *lud)
 			// It's a case instead of an 'if' becuase I have a 
 			// feeling there are more affected models either
 			// on the market or on the way.
-			case 695: break;
+			case 695: break;   // Venture HC
+			case 941: break;   // Venture HC, Japanese version.
+			case 957: break;   // Legend H
+			case 285: break;   // GPSMap 276C/4.80
+			case 402: break;   // GPSMap 396C/4.50
+			case 1095: break;  // GPS72H/2.30
 			default: gusb_syncup();
 		}
 		return;
@@ -359,20 +376,17 @@ int garmin_usb_scan(libusb_unit_data *lud, int req_unit_number)
 		struct usb_device *dev;
 
 		for (dev = bus->devices; dev; dev = dev->next) {
-			/* Probably too promiscious of a match, but since
-			 * Garmin doesn't document the _proper_ matching,
-			 * we just take the easy way out for now.
-			 * Unfortunatey, blowing on DeviceClass == Mass storage
-			 * doesn't work on CO, at least.
+			/* 
+			 * Exclude Mass Storage devices (CO, OR, Nuvi, etc.) 
+			 * from this scan.
+			 * At least on Mac, bDeviceClass isn't 
+			 * USB_MASS_STORAGE as it should be (perhaps because
+			 * the storage driver has already bound to it?) so
+			 * we fondle only the proprietary class devices.
 			 */
 			if (dev->descriptor.idVendor == GARMIN_VID && 
-								dev->config) {
-				switch (dev->descriptor.idProduct) {
-					case 0x19:  // Nuvi;
-					case 0x2244:  // Zumo;
-					case 0x2295:  // CO;
-						continue;
-				}
+			    dev->config &&
+				dev->descriptor.bDeviceClass == USB_CLASS_VENDOR_SPEC ) {
 				if (req_unit_number < 0) {
 					garmin_usb_start(dev, lud);	
 					/* 
