@@ -24,6 +24,7 @@
 #include "cet_util.h"
 #include "csv_util.h"
 #include "inifile.h"
+#include "session.h"
 #include <ctype.h>
 #include <signal.h>
 
@@ -41,7 +42,7 @@ typedef struct arg_stack_s {
 static arg_stack_t
 *push_args(arg_stack_t *stack, const int argn, const int argc, char *argv[])
 {
-	arg_stack_t *res = xmalloc(sizeof(*res));
+	arg_stack_t *res = (arg_stack_t *) xmalloc(sizeof(*res));
 	
 	res->prev = stack;
 	res->argn = argn;
@@ -97,13 +98,13 @@ load_args(const char *filename, int *argc, char **argv[])
 	}
 	gbfclose(fin);
 	
-	argv2 = xmalloc(2 * sizeof(*argv2));
+	argv2 = (char **) xmalloc(2 * sizeof(*argv2));
 	argv2[0] = xstrdup(*argv[0]);
 	argc2 = 1;
 	
 	str = csv_lineparse(line, " ", "\"", 0);
 	while (str != NULL) {
-		argv2 = xrealloc(argv2, (argc2 + 2) * sizeof(*argv2));
+		argv2 = (char **) xrealloc(argv2, (argc2 + 2) * sizeof(*argv2));
 		argv2[argc2] = xstrdup(str);
 		argc2++;
 		str = csv_lineparse(NULL, " ", "\"", 0);
@@ -123,8 +124,8 @@ usage(const char *pname, int shorter )
 			gpsbabel_version );
 	printf(
 "Usage:\n"
-"    %s [options] -i INTYPE -f INFILE -o OUTTYPE -F OUTFILE\n"
-"    %s [options] -i INTYPE -o OUTTYPE INFILE [OUTFILE]\n"
+"    %s [options] -i INTYPE -f INFILE [filter] -o OUTTYPE -F OUTFILE\n"
+"    %s [options] -i INTYPE -o OUTTYPE INFILE [filter] OUTFILE\n"
 "\n"
 "    Converts GPS route and waypoint data from one format type to another.\n"
 "    The input type and filename are specified with the -i INTYPE\n"
@@ -150,7 +151,7 @@ usage(const char *pname, int shorter )
 "    -b               Process command file (batch mode)\n"
 "    -c               Character set for next operation\n"
 "    -N               No smart icons on output\n"
-"    -x filtername    Invoke filter (place between inputs and output) \n"
+"    -x filtername    Invoke filter (placed between inputs and output) \n"
 "    -D level         Set debug level [%d]\n"
 "    -l               Print GPSBabel builtin character sets and exit\n"
 "    -h, -?           Print detailed help and exit\n"
@@ -206,7 +207,7 @@ print_extended_info(void)
 #endif
 #if HAVE_LIBEXPAT
 	"HAVE_LIBEXPAT "
-#if XML_UNICODE
+#if defined(XML_UNICODE)
 	"XML_UNICODE "
 #endif
 #endif
@@ -259,7 +260,10 @@ main(int argc, char *argv[])
 		global_opts.inifile = inifile_init(NULL, MYNAME);
 	}
 	
+	init_vecs();
+	init_filter_vecs();
 	cet_register();
+	session_init();
 	waypt_init();
 	route_init();
 
@@ -315,6 +319,9 @@ main(int argc, char *argv[])
 				optarg = argv[argn][2]
 					? argv[argn]+2 : argv[++argn];
 				ivecs = find_vec(optarg, &ivec_opts);
+				if (ivecs == NULL) {
+					fatal ("Input type '%s' not recognized\n", optarg);
+				}
 				break;
 			case 'o':
 				if (ivecs == NULL) {
@@ -349,6 +356,7 @@ main(int argc, char *argv[])
 			
 				cet_convert_init(ivecs->encode, ivecs->fixed_encode);	/* init by module vec */
 
+				start_session(ivecs->name, fname);
 				ivecs->rd_init(fname);
 				ivecs->read();
 				ivecs->rd_deinit();
@@ -568,6 +576,10 @@ main(int argc, char *argv[])
 					
 		cet_convert_init(ivecs->encode, 1);
 
+		start_session(ivecs->name, argv[0]);
+		if (ivecs->rd_init == NULL) {
+			fatal ("Format does not support reading.\n");
+		}
 		ivecs->rd_init(argv[0]);
 		ivecs->read();
 		ivecs->rd_deinit();
@@ -579,6 +591,10 @@ main(int argc, char *argv[])
 		{
 			cet_convert_init(ovecs->encode, 1);
 			cet_convert_strings(NULL, global_opts.charset, NULL);
+
+			if (ovecs->wr_init == NULL) {
+				fatal ("Format does not support writing.\n");
+			}
 			
 			ovecs->wr_init(argv[1]);
 			ovecs->write();
@@ -614,12 +630,20 @@ main(int argc, char *argv[])
 	 */
 	if (global_opts.masked_objective & POSNDATAMASK) {
 
+                if (!ivecs) {
+                  fatal("Realtime tracking (-T) requires an input type (-t)i such as Garmin or NMEA.\n");
+                }
+
 		if (!ivecs->position_ops.rd_position) {
 			fatal("Realtime tracking (-T) is not suppored by this input type.\n");
 		}
 
 
 		if (ivecs->position_ops.rd_init) {
+			if (!fname) {
+			       fatal("An input file (-f) must be specified.\n");
+			}
+			start_session(ivecs->name, fname);
 			ivecs->position_ops.rd_init(fname);
 		}
 
@@ -668,9 +692,10 @@ main(int argc, char *argv[])
 		if (ivecs->position_ops.rd_deinit) {
 			ivecs->position_ops.rd_deinit();
 		}
-		if (ovecs->position_ops.wr_deinit) {
+		if (ovecs && ovecs->position_ops.wr_deinit) {
 			ovecs->position_ops.wr_deinit();
 		}
+		exit(0);
 	}
 
 
@@ -680,6 +705,7 @@ main(int argc, char *argv[])
 	cet_deregister();
 	waypt_flush_all();
 	route_flush_all();
+	session_exit();
 	exit_vecs();
 	exit_filter_vecs();
 	inifile_done(global_opts.inifile);
@@ -694,4 +720,3 @@ void signal_handler(int sig)
 {
 	tracking_status.request_terminate = 1;
 }
-
