@@ -24,6 +24,10 @@
 #include <stdlib.h>
 
 #include "defs.h"
+#include "cet.h"
+#include "cet_util.h"
+
+#define MYNAME	"mkshort"
 
 static const char vowels[] = "aeiouAEIOU";
 
@@ -50,6 +54,7 @@ typedef struct {
 	unsigned int whitespaceok:1;
 	unsigned int repeating_whitespaceok:1;
 	unsigned int must_uniq:1;
+	unsigned int is_utf8:1;
 } mkshort_handle;
 
 typedef struct {
@@ -96,7 +101,7 @@ mkshort_new_handle()
 #endif
 {
 	int i;
-	mkshort_handle *h = xxcalloc(sizeof *h, 1, file, line);
+	mkshort_handle *h = (mkshort_handle *) xxcalloc(sizeof *h, 1, file, line);
 
 	for (i = 0; i < PRIME; i++)
 		QUEUE_INIT(&h->namelist[i]);
@@ -106,6 +111,7 @@ mkshort_new_handle()
 	h->target_len = DEFAULT_TARGET_LEN;
 	h->must_uniq = 1;
 	h->defname = xstrdup("WPT");
+	h->is_utf8 = (global_opts.charset == &cet_cs_vec_utf8);
 
 	return h;
 }
@@ -132,7 +138,7 @@ void
 add_to_hashlist(mkshort_handle *h, char *name)
 {
 	int hash = hash_string(name);
-	uniq_shortname *s = xcalloc(1, sizeof (uniq_shortname));
+	uniq_shortname *s = (uniq_shortname*) xcalloc(1, sizeof (uniq_shortname));
 
 	s->orig_shortname = xstrdup(name);
 	ENQUEUE_TAIL(&h->namelist[hash], &s->list);
@@ -153,7 +159,7 @@ mkshort_add_to_list(mkshort_handle *h, char *name)
 		dl = sprintf(tbuf, ".%d", s->conflictctr);
 
 		if (l + dl < h->target_len) {
-			name = xrealloc(name, l + dl + 1);
+			name = (char *) xrealloc(name, l + dl + 1);
 			strcat(name, tbuf);
 		}
 		else {
@@ -367,6 +373,16 @@ setshort_mustuniq(short_handle h, int i)
 	hdl->must_uniq = i;
 }
 
+/* 
+ *  Declare that actually characters are (or are not) encoded in UTF-8.
+ */
+void
+setshort_is_utf8(short_handle h, const int is_utf8)
+{
+	mkshort_handle *hdl = (mkshort_handle *) h;
+	hdl->is_utf8 = is_utf8;
+}
+
 char *
 #ifdef DEBUG_MEM
 MKSHORT(short_handle h, const char *istring, DEBUG_PARAMS )
@@ -374,14 +390,16 @@ MKSHORT(short_handle h, const char *istring, DEBUG_PARAMS )
 mkshort(short_handle h, const char *istring)
 #endif
 {
-	char *ostring = xxstrdup(istring, file, line);
+	char *ostring;
 	char *nstring;
 	char *tstring;
 	char *cp;
 	char *np;
 	int i, l, nlen, replaced;
 	mkshort_handle *hdl = (mkshort_handle *) h;
-
+	
+	if (hdl->is_utf8) ostring = cet_utf8_strdup(istring); /* clean UTF-8 string */
+	else ostring = xxstrdup(istring, file, line);
 
 	/*
 	 * A rather horrible special case hack.
@@ -454,6 +472,9 @@ mkshort(short_handle h, const char *istring)
 			continue;
 		if (hdl->goodchars && (!strchr(hdl->goodchars, tstring[i])))
 			continue;
+// FIXME(robertl): we need a way to not return partial UTF-8, but this isn't it.
+//		if (!isascii(tstring[i]))
+//			continue;
 		*cp++ = tstring[i];
 	}
 	*cp = 0;
@@ -522,8 +543,16 @@ mkshort(short_handle h, const char *istring)
 	 * numeric data.
 	 * If the numeric component alone is longer than our target string
 	 * length, use only what'll fit.
- 	 */
-	if ((/*i = */strlen(ostring)) > hdl->target_len) {
+	 */
+	if (hdl->is_utf8) {
+		/* ToDo: Keep trailing numeric data as described above! */
+		if (cet_utf8_strlen(ostring) > hdl->target_len) {
+			char *tmp = cet_utf8_strndup(ostring, hdl->target_len);
+			xfree(ostring);
+			ostring = tmp;
+		}
+	}
+	else if ((/*i = */strlen(ostring)) > hdl->target_len) {
 		char *dp = &ostring[hdl->target_len] - strlen(np);
 		if (dp < ostring) dp = ostring;
 		memmove(dp, np, strlen(np));
@@ -558,7 +587,7 @@ mkshort_from_wpt(short_handle h, const waypoint *wpt)
 	 * which contains placer name, diff, terr, and generally way
 	 * more stuff than should be in any one field...
  	 */
-	if (wpt->gc_data.diff && wpt->gc_data.terr && 
+	if (wpt->gc_data->diff && wpt->gc_data->terr && 
 		wpt->notes && wpt->notes[0]) {
 		return mkshort(h, wpt->notes);
 	}
@@ -572,7 +601,10 @@ mkshort_from_wpt(short_handle h, const waypoint *wpt)
 	}
 
 	/* Should probably never actually happen... */
-	return NULL;
+	/* O.K.: But this can happen (waypoints transformed from trackpoints )! */
+	/*       Now we return every time a valid entity." */
+
+	return mkshort(h, wpt->shortname);
 }
 
 
