@@ -80,8 +80,8 @@ arglist_t raymarine_args[] =
 /* Bitmaps */
 
 typedef struct {
-	char *name;
-	char *mps_name;
+	const char *name;
+	const char *mps_name;
 } raymarine_symbol_mapping_t;
 
 static raymarine_symbol_mapping_t raymarine_symbols[] = {
@@ -167,6 +167,7 @@ static void
 raymarine_rd_init(const char *fname)
 {
 	fin = inifile_init(fname, MYNAME);
+	if (fin->unicode) cet_convert_init(CET_CHARSET_UTF8, 1);
 }
 
 static void
@@ -387,7 +388,7 @@ static void
 write_route_wpt_cb(const waypoint *wpt)
 {
 	int i;
-	static char *items[] = {
+	static const char *items[] = {
 		"Cog",
 		"Eta",
 		"Length",
