@@ -42,8 +42,8 @@
 
 #define MYNAME "TomTom"
 
-static FILE *file_in;
-static FILE *file_out;
+static gbfile *file_in;
+static gbfile *file_out;
 
 static
 arglist_t tomtom_args[] = {
@@ -53,78 +53,182 @@ arglist_t tomtom_args[] = {
 static void
 rd_init(const char *fname)
 {
-	file_in = xfopen(fname, "rb", MYNAME);
+	file_in = gbfopen_le(fname, "rb", MYNAME);
 }
 
 static void
 rd_deinit(void)
 {
-	fclose(file_in);
+	gbfclose(file_in);
 }
 
 static void
 wr_init(const char *fname)
 {
-	file_out = xfopen(fname, "wb", MYNAME);
+	file_out = gbfopen_le(fname, "wb", MYNAME);
 }
 
 static void
 wr_deinit(void)
 {
-	fclose(file_out);
+	gbfclose(file_out);
 }
 
-static unsigned long
-read_long(FILE * f)
+#define read_long(f) gbfgetint32((f))
+#define read_char(f) gbfgetc((f))
+
+/*
+ *  Decode a type 8 compressed record
+ */
+char *
+decode_8(int sz, const unsigned char *inbuf)
 {
-        gbuint32 result = 0;
-	
-        fread(&result, sizeof (result), 1, f);
-        return le_read32(&result);
+  static const char encoding_8[32] = "X. SaerionstldchumgpbkfzvACBMPG-";
+  static const int encoding_8_high[8] = {0x2,0x3,0x4,0x5,0x6,0x7,0xe,0xf};
+
+  // Maximally sized for laziness.
+  char *rval = xmalloc(sz * 3 + 1);
+  char *out = rval;
+
+  int i;
+    for (i = 0; i < sz;) {
+     if (inbuf[0] & 0x80) {
+      int idx;
+      int res;
+      idx = (inbuf[0] & 0x70) >> 4;
+      res = inbuf[0] & 0x0f;
+      res |= encoding_8_high[idx] << 4;
+
+      *out++ = res;
+
+      inbuf++;
+      i++;
+     } else {
+      int c1 = (inbuf[0] & 0x7c) >> 2;
+      int c2 = ((inbuf[0] & 3) << 3) | (inbuf[1] & 0xe0) >> 5;
+      int c3 = inbuf[1] &  0x1f;
+      if ((c1 | c2 | c3) > 0x1f) fatal("bit unpacking error");
+      *out++ = encoding_8[c1];
+      *out++ = encoding_8[c2];
+      *out++ = encoding_8[c3];
+      inbuf+=2;
+      i+=2;
+    }
+  }
+  return rval;
 }
 
-static unsigned char
-read_char( FILE *f)
+void
+decode_latlon(double *lat, double *lon)
 {
-	unsigned char result = 0;
-	fread( &result, 1, 1, f );
-	return result;
+  unsigned char latbuf[3];
+  unsigned char lonbuf[3];
+  double rlat, rlon;
+
+  gbfread(&lonbuf, 3, 1, file_in );
+  gbfread(&latbuf, 3, 1, file_in );
+  rlat = ((latbuf[2] << 16) + (latbuf[1] << 8) + latbuf[0]) / 1000000.0;
+
+  *lat = 80 - rlat;
+  *lon = rlon = 123.456;
+
+}
+
+static void
+check_recsize(int sz)
+{
+	if ((sz > 100000) || (sz < 0)) {
+		fatal(MYNAME ":malformed file.  Bad record size.");
+	}
 }
 
 static void
 data_read(void)
 {
-	unsigned char rectype;
+	int rectype;
 	long recsize;
 	long x;
 	long y;
 	char *desc;
 	waypoint *wpt_tmp;
-	while (!feof( file_in ) ) {
+	while (!gbfeof( file_in ) ) {
 		rectype = read_char( file_in );
-		if ( rectype == 1 ) {
+                if (rectype == EOF) {
+			fatal(MYNAME ":Unexpected EOF.");
+                }
+		if (global_opts.debug_level >= 5)
+			printf("Reading record type %d\n", rectype );
+                switch (rectype) {
+                  case 0: 
+                  case 100:
+			if (global_opts.debug_level >= 5)
+				printf("Skipping deleted record\n" );
+			recsize = read_long( file_in ) - 5;
+                        check_recsize(recsize);
+			if (global_opts.debug_level >= 5)
+				printf("Skipping %li bytes\n", recsize );
+                        gbfseek(file_in, recsize, SEEK_CUR);
+                        break;
+		  case 1:
 			/* a block header; ignored on read */
 			read_long( file_in );
 			read_long( file_in );
 			read_long( file_in );
 			read_long( file_in );
 			read_long( file_in );
-		}
-		else if ( rectype == 2 || rectype == 3 ) {
+		        break;
+                  case 2:
+                  case 3:
 			recsize = read_long( file_in );
+                        check_recsize(recsize);
 			x = read_long( file_in );
 			y = read_long( file_in );
 			desc = (char *)xmalloc( recsize - 13 );
-			fread( desc, recsize-13, 1, file_in );
+			gbfread( desc, recsize-13, 1, file_in );
 			
 			wpt_tmp = waypt_new();
 
 			wpt_tmp->longitude = x/100000.0;
 			wpt_tmp->latitude = y/100000.0;
 			wpt_tmp->description = desc;
+			// TODO:: description in rectype 3 contains two zero-terminated strings
+			// First is same as rectype 2, second apparently contains the unique ID of the waypoint
+			// See http://www.tomtom.com/lib/doc/PRO/TTN6_SDK_documentation.zip
+			if ( rectype == 3) {
+				warning("Unexpected waypoint record type %d encountered.\nThe unique ID of the POI may have been dropped.\n", rectype );
+			}
 
 			waypt_add(wpt_tmp);
+		  break;
+              case 8:
+              case 24:
+#if 0 // Fallthrough for now to silently ignore these until this is done.
+                recsize = read_char( file_in ) ;
+                check_recsize(recsize);
+		wpt_tmp = waypt_new();
+                decode_latlon(&wpt_tmp->latitude, &wpt_tmp->longitude);
+                gbfread( tbuf, 3, 1, file_in );
+                gbfread( tbuf, 3, 1, file_in );
+                gbfread( tbuf, recsize, 1, file_in );
+                wpt_tmp->shortname = decode_8(recsize, tbuf);
+                waypt_add(wpt_tmp);
+                break;
+#else
+#endif
+              case 9:
+              case 25:
+                recsize = read_char( file_in ) + 6;
+                check_recsize(recsize);
+                if (global_opts.debug_level >= 5)
+                  warning("Unknown record type 0x%x; skipping %ld bytes.\n",
+                          rectype, recsize);
+                 gbfseek(file_in, recsize, SEEK_CUR);
+                 break;
+          default:
+                if (global_opts.debug_level >= 1) {
+			warning("Unexpected waypoint record type: %d at offset 0x%x\n", rectype, gbftell(file_in) );
 		}
+                }
 	}
 }
 
@@ -175,31 +279,17 @@ compare_lon(const void *a, const void *b)
 	return compare_lat(a,b);
 }
 
-static void 
-write_long( FILE *file, long value ) {
-	 gbuint32 tmp = 0;
-	 le_write32( &tmp, value );
-		 
-	 fwrite( &tmp, sizeof(tmp), 1, file );
-} 
+#define write_long(f,v) gbfputint32((v),f)
 
 static void
-write_float_as_long( FILE *file, double value ) 
+write_float_as_long( gbfile *file, double value )
 {
 	long tmp = (value + 0.500000000001);
 	write_long( file, tmp);
 }
 
-static void
-write_char( FILE *file, unsigned char value ) {
-	fwrite( &value, 1, 1, file );
-}
-
-static void
-write_string( FILE *file, char *str ) {
-	fwrite( str, strlen(str), 1, file );
-	write_char( file, '\0' );
-}
+#define write_char(f,c) gbfputc((c),f)
+#define write_string(f,s) gbfputcstr((s),f)
 
 struct blockheader {
 	struct hdr *start;
@@ -214,7 +304,7 @@ struct blockheader {
 };
 
 static void
-write_blocks( FILE *f, struct blockheader *blocks ) {
+write_blocks( gbfile *f, struct blockheader *blocks ) {
 	int i;
 	write_char( f, 1 );
 	write_long( f, blocks->size );
@@ -230,11 +320,26 @@ write_blocks( FILE *f, struct blockheader *blocks ) {
 	}
 	if ( !blocks->ch1 && !blocks->ch2 ) {
 		for ( i = 0; i < blocks->count; i++ ) {
+            		char desc_field [256];
 			write_char( f, 2 );
-			write_long( f, strlen( blocks->start[i].wpt->description ) + 14 );
+            if (global_opts.smart_names && 
+	      		blocks->start[i].wpt->gc_data->diff && 
+			blocks->start[i].wpt->gc_data->terr) {
+                snprintf(desc_field,sizeof(desc_field),"%s(t%ud%u)%s(type%dcont%d)",blocks->start[i].wpt->description,
+                blocks->start[i].wpt->gc_data->terr/10,
+                blocks->start[i].wpt->gc_data->diff/10,
+                blocks->start[i].wpt->shortname,
+                (int) blocks->start[i].wpt->gc_data->type,
+                (int) blocks->start[i].wpt->gc_data->container);
+                //Unfortunately enums mean we get numbers for cache type and container.
+            } else {
+                snprintf(desc_field, sizeof(desc_field), "%s",
+			blocks->start[i].wpt->description);
+            }
+			write_long( f, strlen( desc_field ) + 14 );
 			write_float_as_long( f, blocks->start[i].wpt->longitude*100000);
 			write_float_as_long( f, blocks->start[i].wpt->latitude*100000);
-			write_string( f, blocks->start[i].wpt->description);
+			write_string( f, desc_field);
 		}
 	}
 }	
@@ -354,5 +459,5 @@ ff_vecs_t tomtom_vecs = {
 	data_write,
 	NULL,
 	tomtom_args,
-	CET_CHARSET_ASCII, 0	/* CET-REVIEW */
+	CET_CHARSET_MS_ANSI, 0	/* CET-REVIEW */
 };
