@@ -45,6 +45,7 @@ extern ff_vecs_t compegps_vecs;
 extern ff_vecs_t copilot_vecs;
 extern ff_vecs_t coto_vecs;
 extern ff_vecs_t cst_vecs;
+extern ff_vecs_t delbin_vecs;
 extern ff_vecs_t dg100_vecs;
 extern ff_vecs_t easygps_vecs;
 extern ff_vecs_t garmin_vecs;
@@ -68,6 +69,7 @@ extern ff_vecs_t HsaEndeavourNavigator_vecs;
 extern ff_vecs_t html_vecs;
 extern ff_vecs_t igc_vecs;
 extern ff_vecs_t ignr_vecs;
+extern ff_vecs_t igo8_vecs;
 extern ff_vecs_t kml_vecs;
 extern ff_vecs_t lowranceusr_vecs;
 extern ff_vecs_t mag_fvecs;
@@ -81,6 +83,8 @@ extern ff_vecs_t mps_vecs;
 extern ff_vecs_t msroute_vecs;
 extern ff_vecs_t mtk_vecs;
 extern ff_vecs_t mtk_fvecs;
+extern ff_vecs_t mtk_m241_vecs;
+extern ff_vecs_t mtk_m241_fvecs;
 extern ff_vecs_t navicache_vecs;
 extern ff_vecs_t netstumbler_vecs;
 extern ff_vecs_t nmea_vecs;
@@ -95,6 +99,8 @@ extern ff_vecs_t psp_vecs;
 extern ff_vecs_t quovadis_vecs;
 extern ff_vecs_t saroute_vecs;
 extern ff_vecs_t shape_vecs;
+extern ff_vecs_t skytraq_vecs;
+extern ff_vecs_t skytraq_fvecs;
 #if CSVFMTS_ENABLED
 extern ff_vecs_t stmsdf_vecs;
 #endif
@@ -137,6 +143,36 @@ extern ff_vecs_t destinator_trl_vecs;
 extern ff_vecs_t destinator_itn_vecs;
 extern ff_vecs_t exif_vecs;
 extern ff_vecs_t vidaone_vecs;
+extern ff_vecs_t gopal_vecs;
+extern ff_vecs_t humminbird_vecs;
+extern ff_vecs_t humminbird_ht_vecs;
+extern ff_vecs_t mapasia_tr7_vecs;
+extern ff_vecs_t gnav_trl_vecs;
+extern ff_vecs_t navitel_trk_vecs;
+extern ff_vecs_t ggv_ovl_vecs;
+#if CSVFMTS_ENABLED
+extern ff_vecs_t jtr_vecs;
+#endif
+extern ff_vecs_t itracku_vecs;
+extern ff_vecs_t itracku_fvecs;
+extern ff_vecs_t sbp_vecs;
+extern ff_vecs_t ng_vecs;
+extern ff_vecs_t sbn_vecs;
+extern ff_vecs_t mmo_vecs;
+extern ff_vecs_t bushnell_vecs;
+extern ff_vecs_t bushnell_trl_vecs;
+extern ff_vecs_t skyforce_vecs;
+extern ff_vecs_t v900_vecs;
+extern ff_vecs_t pocketfms_bc_vecs;
+extern ff_vecs_t pocketfms_fp_vecs;
+extern ff_vecs_t pocketfms_wp_vecs;
+extern ff_vecs_t enigma_vecs;
+extern ff_vecs_t vpl_vecs;
+extern ff_vecs_t teletype_vecs;
+extern ff_vecs_t jogmap_vecs;
+extern ff_vecs_t wintec_tes_vecs;
+extern ff_vecs_t subrip_vecs;
+extern ff_vecs_t format_garmin_xt_vecs;
 
 static
 vecs_t vec_list[] = {
@@ -185,6 +221,18 @@ vecs_t vec_list[] = {
 		"Garmin serial/USB protocol", 
 		NULL
 	},
+	{
+		&gdb_vecs,
+		"gdb",
+		"Garmin MapSource - gdb",
+		"gdb"
+	},
+	{
+		&gtc_vecs,
+		"gtrnctr",
+		"Garmin Training Center",
+		"xml"
+	},
 	{
 		&mapsend_vecs,
 		"mapsend",
@@ -203,19 +251,25 @@ vecs_t vec_list[] = {
 		"NMEA 0183 sentences",
 		NULL
 	},
-        {
-                &kml_vecs,
-                "kml",
-                "Google Earth (Keyhole) Markup Language",
-                "kml"
+	{
+		&ozi_vecs,
+		"ozi",
+		"OziExplorer",
+		NULL
 	},
-#if MAXIMAL_ENABLED
 	{
 		&pcx_vecs,
 		"pcx",
 		"Garmin PCX5",
 		"pcx"
 	},
+	{
+		&kml_vecs,
+		"kml",
+		"Google Earth (Keyhole) Markup Language",
+		"kml"
+	},
+#if MAXIMAL_ENABLED
 	{
 		&gpsutil_vecs,
 		"gpsutil",
@@ -308,14 +362,14 @@ vecs_t vec_list[] = {
 		&easygps_vecs,
 		"easygps",
 		"EasyGPS binary format",
-		".loc"
+		"loc"
 	},
 #if PDBFMTS_ENABLED
 	{
 		&quovadis_vecs,
 		"quovadis",
 		"Quovadis",
-		"pdb"	
+		"pdb"
 	},
 	{
 		&gpilots_vecs,
@@ -370,12 +424,6 @@ vecs_t vec_list[] = {
 		"DeLorme GPL",
 		"gpl"
 	},
-	{
-		&ozi_vecs,
-		"ozi",
-		"OziExplorer",
-		NULL
-	},
 	{
 		&text_vecs,
 		"text",
@@ -386,7 +434,7 @@ vecs_t vec_list[] = {
 		&html_vecs,
 		"html",
 		"HTML Output",
-		"html"	
+		"html"
 	},
 #if PDBFMTS_ENABLED
 	{
@@ -432,12 +480,32 @@ vecs_t vec_list[] = {
                 "MTK Logger (iBlue 747,...) Binary File Format",
                 "bin"
         },
+        {
+                &mtk_m241_vecs,
+                "m241",
+                "Holux M-241 (MTK based) download",
+                NULL
+        },
+        {
+                &mtk_m241_fvecs,
+                "m241-bin",
+                "Holux M-241 (MTK based) Binary File Format",
+                "bin"
+        },
+#endif // MAXIMAL_ENABLED
         {
                 &wbt_svecs,
                 "wbt",
                 "Wintec WBT-100/200 GPS Download",
                 NULL
         },
+		{
+				&vpl_vecs,
+				"vpl",
+				"Honda/Acura Navigation System VP Log File Format",
+				NULL
+		},
+#if MAXIMAL_ENABLED
         {
                 &wbt_fvecs,
                 "wbt-bin",
@@ -472,7 +540,7 @@ vecs_t vec_list[] = {
 	{
 		&overlay_vecs,
 		"overlay",
-		"GeoGrid-Viewer",
+		"Geogrid-Viewer",
 		"ovl"
 	},
 #endif
@@ -519,25 +587,19 @@ vecs_t vec_list[] = {
 		"vitosmt",
 		"Vito Navigator II tracks",
 		"smt"
-	},	
+	},
 	{
 		&wfff_xml_vecs,
 		"wfff",
 		"WiFiFoFum 2.0 for PocketPC XML",
 		"xml"
 	},
-	{
-		&gdb_vecs,
-		"gdb",
-		"Garmin MapSource - gdb",
-		"gdb"
-	},	
 	{
 		&bcr_vecs,
 		"bcr",
 		"Motorrad Routenplaner (Map&Guide) .bcr files",
 		"bcr"
-	},	
+	},
 #if PDBFMTS_ENABLED
 	{
 		&coto_vecs,
@@ -649,7 +711,7 @@ vecs_t vec_list[] = {
 	{
 		&gtc_vecs,
 		"gtrnctr",
-		"Garmin Training Center"
+		"Garmin Training Center (.tcx)",
 		"xml"
 	},
 	{
@@ -685,7 +747,7 @@ vecs_t vec_list[] = {
         {
                 &ggv_log_vecs,
                 "ggv_log",
-                "Geogrid Viewer tracklogs (.log)",
+                "Geogrid-Viewer tracklogs (.log)",
                 "log"
         },
 #if CSVFMTS_ENABLED
@@ -717,7 +779,7 @@ vecs_t vec_list[] = {
         {
                 &xol_vecs,
                 "xol",
-                "Swiss Map # (.xol) format",
+                "Swiss Map 25/50/100 (.xol)",
                 "xol"
         },
         {
@@ -774,6 +836,195 @@ vecs_t vec_list[] = {
                 "VidaOne GPS for Pocket PC (.gpb)",
 		"gpb"
         },
+	{
+		&igo8_vecs,
+		"igo8",
+		"IGO8 .trk",
+		"trk"
+	},
+	{
+                &gopal_vecs,
+                "gopal",
+                "GoPal GPS track log (.trk)",
+		"trk"
+        },
+	{
+		&humminbird_vecs,
+		"humminbird",
+		"Humminbird waypoints and routes (.hwr)",
+		"hwr"
+        },
+	{
+		&humminbird_ht_vecs,
+		"humminbird_ht",
+		"Humminbird tracks (.ht)",
+		"ht"
+        },
+	{
+		&mapasia_tr7_vecs,
+		"mapasia_tr7",
+		"MapAsia track file (.tr7)",
+		"tr7"
+	},
+	{
+		&gnav_trl_vecs,
+		"gnav_trl",
+		"Google Navigator Tracklines (.trl)",
+		"trl"
+	},
+	{
+		&navitel_trk_vecs,
+		"navitel_trk",
+		"Navitel binary track (.bin)",
+		"bin"
+	},
+        {
+                &ggv_ovl_vecs,
+                "ggv_ovl",
+                "Geogrid-Viewer ascii overlay file (.ovl)",
+                "ovl"
+        },
+#if CSVFMTS_ENABLED
+        {
+                &jtr_vecs,
+                "jtr",
+                "Jelbert GeoTagger data file",
+                "jtr"
+        },
+#endif
+        {
+                &itracku_vecs,
+                "itracku",
+                "XAiOX iTrackU Logger",
+                NULL
+        },
+
+        {
+                &itracku_fvecs,
+                "itracku-bin",
+                "XAiOX iTrackU Logger Binary File Format",
+                "bin"
+        },
+        {
+                &sbp_vecs,
+                "sbp",
+                "NaviGPS GT-31/BGT-31 datalogger (.sbp)",
+                "sbp"
+        },
+        {
+                &sbn_vecs,
+                "sbn",
+                "NaviGPS GT-31/BGT-31 SiRF binary logfile (.sbn)",
+                "sbn"
+        },
+        {
+                &mmo_vecs,
+                "mmo",
+                "Memory-Map Navigator overlay files (.mmo)",
+                "mmo"
+        },
+        {
+                &bushnell_vecs,
+                "bushnell",
+                "Bushnell GPS Waypoint file",
+                "wpt"
+        },
+        {
+                &bushnell_trl_vecs,
+                "bushnell_trl",
+                "Bushnell GPS Trail file",
+                "trl"
+        },
+        {
+        	&skyforce_vecs,
+        	"skyforce",
+        	"Skymap / KMD150 ascii files",
+        	NULL
+        },
+        {
+        	&pocketfms_bc_vecs,
+        	"pocketfms_bc",
+        	"PocketFMS breadcrumbs",
+        	NULL
+        },
+        {
+        	&pocketfms_fp_vecs,
+        	"pocketfms_fp",
+        	"PocketFMS flightplan (.xml)",
+        	"xml"
+        },
+        {
+        	&pocketfms_wp_vecs,
+        	"pocketfms_wp",
+        	"PocketFMS waypoints (.txt)",
+        	"txt"
+        },
+        {
+        	&v900_vecs,
+        	"v900",
+        	"Columbus/Visiontac V900 files (.csv)",
+        	NULL
+        },
+        {
+        	&ng_vecs,
+        	"naviguide",
+        	"Naviguide binary route file (.twl)",
+        	"twl"
+        },
+        {
+        	&enigma_vecs,
+        	"enigma",
+        	"Enigma binary waypoint file (.ert)",
+        	"ert"
+        },
+	{
+		&delbin_vecs, 
+		"delbin",
+		"DeLorme PN-20/PN-30/PN-40 USB protocol",
+		NULL
+	}, 
+        {
+                &skytraq_vecs,
+                "skytraq",
+                "SkyTraq Venus based loggers (download)",
+                NULL
+        },
+        {
+                &teletype_vecs,
+                "teletype",
+                "Teletype [ Get Jonathon Johnson to describe",
+                NULL
+        },
+        {
+                &skytraq_fvecs,
+                "skytraq-bin",
+                "SkyTraq Venus based loggers Binary File Format",
+                "bin"
+        },
+        {
+                &jogmap_vecs,
+                "jogmap",
+                "Jogmap.de XML format",
+                "xml"
+        },
+        {
+                &wintec_tes_vecs,
+                "wintec_tes",
+                "Wintec TES file",
+                "tes"
+        },
+	{
+		&subrip_vecs,
+		"subrip",
+		"SubRip subtitles for video mapping (.srt)",
+		"srt"
+	},
+        {
+        	&format_garmin_xt_vecs,
+        	"garmin_xt",
+        	"Mobile Garmin XT Track files",
+        	NULL
+        },
 #endif // MAXIMAL_ENABLED
 	{
 		NULL,
@@ -783,6 +1034,28 @@ vecs_t vec_list[] = {
 	}
 };
 
+void
+init_vecs(void)
+{
+	vecs_t *vec = vec_list;
+	while ( vec->vec ) {
+		arglist_t *ap;
+		if ( vec->vec->args ) {
+			for ( ap = vec->vec->args; ap->argstring; ap++ ) {
+				ap->argvalptr = NULL;
+				if (ap->argval) *ap->argval = NULL;
+			}
+		}
+		vec++;
+	}
+}
+
+int
+is_integer(const char *c)
+{
+	return isdigit(c[0]) || ((c[0] == '+' || c[0] == '-') && isdigit(c[1]));
+}
+
 void 
 exit_vecs( void )
 {
@@ -796,12 +1069,12 @@ exit_vecs( void )
 			for ( ap = vec->vec->args; ap->argstring; ap++ ) {
 				if ( ap->defaultvalue && 
 					( ap->argtype == ARGTYPE_INT ) &&
-					! isdigit(ap->defaultvalue[0])) {
+					! is_integer(ap->defaultvalue)) {
 					warning("%s: not an integer\n", ap->argstring);
 				}
-				if ( ap->argval && *ap->argval ) {
-					xfree(*ap->argval);
-					*ap->argval = NULL;
+				if ( ap->argvalptr ) {
+					xfree(ap->argvalptr);
+					*ap->argval = ap->argvalptr = NULL;
 				}
 			}
 		}
@@ -813,11 +1086,16 @@ void
 assign_option(const char *module, arglist_t *ap, const char *val)
 {
 	char *c;
-	
-	if (*ap->argval != NULL) {
-		xfree(*ap->argval);
-		*ap->argval = NULL;
+
+	if (ap->argval == NULL)
+		fatal("%s: No local variable defined for option \"%s\"!", module, ap->argstring);
+
+	if (ap->argvalptr != NULL) {
+		xfree(ap->argvalptr);
+		ap->argvalptr = NULL;
 	}
+	if (ap->argval) *ap->argval = NULL;
+
 	if (val == NULL) return;
 
 	if (case_ignore_strcmp(val, ap->argstring) == 0) c = "";
@@ -864,12 +1142,12 @@ assign_option(const char *module, arglist_t *ap, const char *val)
 	}
 
 	/* for bool options without default: don't set argval if "FALSE" */
-	
+
 	if (((ap->argtype & ARGTYPE_TYPEMASK) == ARGTYPE_BOOL) && 
 	    (*c == '0') && (ap->defaultvalue == NULL)) {
 		return;
 	}
-	*ap->argval = xstrdup(c);
+	*ap->argval = ap->argvalptr = xstrdup(c);
 }
 
 void
@@ -917,14 +1195,14 @@ find_vec(char *const vecname, char **opts)
 		
 		if (vec->vec->args) {
 			for (ap = vec->vec->args; ap->argstring; ap++) {
-				char *opt;
+				const char *opt;
 				
 				if ( res ) {
 					opt = get_option(*opts, ap->argstring);
 					if ( opt ) {
 						found = 1;
 						assign_option(svecname, ap, opt);
-						xfree(opt);
+						xfree((char *)opt);
 						continue;
 					}
 				}
@@ -942,9 +1220,10 @@ find_vec(char *const vecname, char **opts)
 			disp_vec_options(vec->name, vec->vec->args);
 		
 #if CSVFMTS_ENABLED		
-		xcsv_setup_internal_style( NULL );
+		// xcsv_setup_internal_style( NULL );
 #endif // CSVFMTS_ENABLED		
 		xfree(v);
+		vec->vec->name = vec->name;	/* needed for session information */
 		return vec->vec;
 		
 	}
@@ -971,14 +1250,14 @@ find_vec(char *const vecname, char **opts)
 		
 		if (vec_list[0].vec->args) {
 			for (ap = vec_list[0].vec->args; ap->argstring; ap++) {
-				char *opt;
+				const char *opt;
 				
 				if ( res ) {
 					opt = get_option(*opts, ap->argstring);
 					if ( opt ) {
 						found = 1;
 						assign_option(svecname, ap, opt);
-						xfree(opt);
+						xfree((char *)opt);
 						continue;
 					}
 				}
@@ -995,12 +1274,12 @@ find_vec(char *const vecname, char **opts)
 
 		if (global_opts.debug_level >= 1)
 			disp_vec_options(svec->name, vec_list[0].vec->args);
-#if CSVFMTS_ENABLED		
+#if CSVFMTS_ENABLED
 		xcsv_setup_internal_style(svec->style_buf);
-#endif // CSVFMTS_ENABLED		
+#endif // CSVFMTS_ENABLED
 
 		xfree(v);
-
+		vec_list[0].vec->name = svec->name;	/* needed for session information */
 		return vec_list[0].vec;
 	}
 	
@@ -1042,11 +1321,14 @@ get_option(const char *iarglist, const char *argname)
 			 * return "bar".   Otherwise, we assume we have
 			 * simply "foo" so we return that.
 			 */
-			if (argp[arglen] == '=')
+            if (argp[arglen] == '=') {
 				rval = argp + arglen + 1;
-			else
+                break;
+            }
+            else if (argp[arglen] == '\0') {
 				rval = argp;
-			break;
+                break;
+            }
 		}
 	}
 	/*
@@ -1069,8 +1351,8 @@ static signed int
 alpha (const void *a, const void *b)
 {
 
-	const vecs_t *const *ap = a;
-	const vecs_t *const *bp = b;
+	const vecs_t *const *ap = (const vecs_t *const*) a;
+	const vecs_t *const *bp = (const vecs_t *const*) b;
 	
 	return case_ignore_strcmp((*ap)->desc , (*bp)->desc);
 }
@@ -1101,7 +1383,7 @@ sort_and_unify_vecs(int *ctp)
 #endif // CSVFMTS_ENABLED
 
 
-	svp = xcalloc(vc, sizeof(style_vecs_t *));
+	svp = (vecs_t **)xcalloc(vc, sizeof(style_vecs_t *));
 	/* Normal vecs are easy; populate the first part of the array. */
 	for (vec = vec_list; vec->vec; vec++, i++) {
 		svp[i] = vec;
@@ -1114,9 +1396,9 @@ sort_and_unify_vecs(int *ctp)
 	/* Walk the style list, parse the entries, dummy up a "normal" vec */
 	for (svec = style_list; svec->name; svec++, i++)  {
 		xcsv_read_internal_style(svec->style_buf);
-		svp[i] = xcalloc(1, sizeof **svp);
+		svp[i] = (vecs_t*) xcalloc(1, sizeof **svp);
 		svp[i]->name = svec->name;
-		svp[i]->vec = xmalloc(sizeof(*svp[i]->vec));
+		svp[i]->vec = (ff_vecs_t*) xmalloc(sizeof(*svp[i]->vec));
 		svp[i]->extension = xcsv_file.extension;
 		*svp[i]->vec = *vec_list[0].vec; /* Interits xcsv opts */
 		/* Reset file type to inherit ff_type from xcsv for everything
@@ -1135,11 +1417,11 @@ sort_and_unify_vecs(int *ctp)
 		switch(xcsv_file.datatype) {
 			case 0:
 			case wptdata:
-				svp[i]->vec->cap[ff_cap_rw_wpt] = ff_cap_read | ff_cap_write; break;
+				svp[i]->vec->cap[ff_cap_rw_wpt] = (ff_cap) (ff_cap_read | ff_cap_write); break;
 			case trkdata:
-				svp[i]->vec->cap[ff_cap_rw_trk] = ff_cap_read | ff_cap_write; break;
+				svp[i]->vec->cap[ff_cap_rw_trk] = (ff_cap) (ff_cap_read | ff_cap_write); break;
 			case rtedata:
-				svp[i]->vec->cap[ff_cap_rw_rte] = ff_cap_read | ff_cap_write; break;
+				svp[i]->vec->cap[ff_cap_rw_rte] = (ff_cap)(ff_cap_read | ff_cap_write); break;
 			default: ;
 		}
 		svp[i]->desc = xcsv_file.description;
@@ -1180,7 +1462,7 @@ disp_vecs(void)
 				(ap->argtype & ARGTYPE_REQUIRED)?"(required)":"");
 		}
 	}
-	xfree (svp);	
+	xfree (svp);
 	return;
 }
 
@@ -1208,7 +1490,7 @@ disp_vec( const char *vecname )
 				(ap->argtype & ARGTYPE_REQUIRED)?"(required)":"");
 		}
 	}
-	xfree (svp);	
+	xfree (svp);
 	return;
 }
 
@@ -1335,7 +1617,7 @@ disp_formats(int version)
 				disp_v3(vec);
 			}
 		}
-		xfree (svp);	
+		xfree (svp);
 		break;
 	default:
 		;
